# Copyright (C) 2021-2022 Xilinx, Inc.  All rights reserved.
# SPDX-License-Identifier: BSD-3-Clause

__author__ = "Hugh Maguire"
__copyright__ = "Copyright 2021, Xilinx"

import sys
RAFT_DIR = '/usr/share/raft/'
sys.path.append(RAFT_DIR + 'xserver/utils')
import Pyro4
import logging
import base64
from cffi import FFI
from utils import ffi, xhelper_handle,  open_c_library, cdata_to_py
from utils import extract_b64_encoded_string, cdata_string_to_py

i2c_handle = open_c_library(RAFT_DIR+"xserver/xcffi/drv_header/pat/xi2c_python.h", "/usr/lib/libXI2c.so.1")

@Pyro4.expose
class I2C(object):
    i2c_dict = {}

    def __init__(self):
        ret = xhelper_handle.XHelper_MetalInit(xhelper_handle.METAL_LOG_ERROR)
        if 0 != ret:
            logging.error("I2C: XHelper_MetalInit failed. ret = ", ret)
        logging.info("Inside I2C Constructor")
        pass

    # System initialization API
    def XI2c_Initialize(self, device):
        """
        API Initialise one instance of an I2c driver.

        :param device: contains the number of the i2c device
        :return: pointer to instance if successful, NULL on error.

        """
        logging.debug("XI2c_Initialize(Init)")
        Inst_ptr = ffi.new("XI2c *")
        ret = i2c_handle.XI2c_Initialize(Inst_ptr, device)
        Inst = cdata_to_py(Inst_ptr[0])
        self.i2c_dict[0] = Inst_ptr
        return ret

    def XI2c_ReadINA226Reg(self, addr, reg):
        """
        API Read a INA226 register.

        :param addr: addr of INA226 device
        :param reg:  register to be read
        :return: ret_code, pointer unsigned short

        """

        Inst_ptr = self.i2c_dict[0];
        Val_ptr = ffi.new("unsigned short *")

        ret = i2c_handle.XI2c_ReadINA226Reg(Inst_ptr, addr, reg, Val_ptr)
        return ret, Val_ptr[0]

    def XI2c_WriteINA226Reg(self, addr, reg, val):

        """
        API Write an INA266 register

        :param addr: addr of INA226 device
        :param reg:  register to be read
        :param val:  value to be written
        :return: ret_code

        """
        Inst_ptr = self.i2c_dict[0];

        ret = i2c_handle.XI2c_WriteINA226Reg(Inst_ptr, addr, reg, val)
        return ret

    # System initialization API
    def XI2c_Release(self):
        """
        API Release an Instance of I2c driver
        """
        Inst_ptr = self.i2c_dict[0]
        logging.debug("XI2c_Release()")
        i2c_handle.XI2c_Release(Inst_ptr)
        return

    def __del__(self):
        logging.info("Inside I2C Destructor")

"""
Some quick test code
"""

#if __name__ == "__main__" :
#        I2c = I2C()
#        I2c.XI2c_Initialize(2)
#        Inst_ptr = I2c.i2c_dict[0]
#        Inst = cdata_to_py(Inst_ptr[0])
#        ret, val_ptr = I2c.XI2c_ReadINA226Reg(0x43, 0x5)
#        print(hex(val_ptr[0]))
#        ret = I2c.XI2c_WriteINA226Reg(0x43, 0x5, 0x1234)
#        ret, val_ptr = I2c.XI2c_ReadINA226Reg(0x43, 0x5)
#        print(hex(val_ptr[0]))
#        pass



# Copyright (C) 2021-2022 Xilinx, Inc.  All rights reserved.
# Copyright (C) 2022-2024 Advanced Micro Devices, Inc. All Rights Reserved.
# SPDX-License-Identifier: BSD-3-Clause

__author__ = "Anish Kadamathikuttiyil Karthikeyan Pillai"
__copyright__ = "Copyright 2021, Xilinx"

import sys
def usage():
    # sys.argv[0] - usage python file name
    # sys.argv[1] - xclient/xcffi/xpyro
    # sys.argv[2] - host/board
    # sys.argv[3] - ip_address
    # sys.argv[4] - port
    # sys.argv[5] - "switchable" option integrated in xsa
    print(f"Usage: The application work in three modes xclient, xcffi and xpyro\n"
          f"python3 {sys.argv[0]} xclient host/board ip_address port switchable => Runs at host or board with xclient\n"
          f"sudo python3 {sys.argv[0]} xcffi switchable => Runs at board with xcffi\n"
          f"python3 {sys.argv[0]} xpyro host/board ip_address port switchable => Runs at host or board with xpyro\n"
          f"Example1: python3 {sys.argv[0]} xclient host 169.254.10.2 9090\n"
          f"Example2: python3 {sys.argv[0]} xclient host 169.254.10.2 9090 switchable ")

switchable_flag = False

# 'xcffi' option will run only in the board
if (len(sys.argv) ==  2):
    if (sys.argv[1] != 'xcffi'):
        usage()
        sys.exit()
elif (len(sys.argv) ==  3):
    if (sys.argv[1] != 'xcffi'):
        usage()
        sys.exit()
    elif (sys.argv[2] == 'switchable'):
            switchable_flag = True
elif (len(sys.argv) ==  5):
    pass
elif (len(sys.argv) ==  6):
    if (sys.argv[5] == 'switchable'):
        switchable_flag = True
else:
    usage()
    sys.exit()

# The 'xclient' option will run both in host and board
if (sys.argv[1] == 'xclient'):
    # For 'xclient' option the path of the xclient is required
    # The relative path in the host is given for host case
    if (sys.argv[2] == 'host'):
        sys.path.append('../../../xclient/dfe')
    # The location of xclient in the filesystem is given for the board case
    elif (sys.argv[2] == 'board'):
        sys.path.append('/usr/share/raft/xclient/dfe')
    else:
        usage()
        sys.exit()
    import ccf_client
    handle = ccf_client.ccf
    ip_address = sys.argv[3]
    port = sys.argv[4]

elif (sys.argv[1] == 'xcffi'):
    # For 'xcffi' option the path of the cffi python code is required
    sys.path.append('/usr/share/raft/xserver/xcffi/drv_api/dfe')
    from ccf_server import CCF
    handle = CCF()

# The 'xpyro' option will run both in host and board
elif (sys.argv[1] == 'xpyro'):
    ip_address = sys.argv[3]
    port = sys.argv[4]
    import Pyro4
    # Prepare the uri needed for pyro communication
    uri = f"PYRO:CCF@{ip_address}:{port}"
    handle = Pyro4.Proxy(uri)
else:
    usage()
    sys.exit()

if (sys.argv[1] == 'xclient'):
    #SetIpAndPort
    handle.SetIpAndPort(ip_address, port)
    #GetPythonLogLevels
    PythonLogLevels = handle.GetPythonLogLevels()
    #SetClientLogLevel
    handle.SetClientLogLevel(PythonLogLevels["DEBUG"])

#Description:
#   Return the logging levels supported by logging library in python
#Input Arguments:
#   None
#Return:
#   Dictionary showing the log levels supported by logging library

#GetPythonLogLevels
PythonLogLevels = handle.GetPythonLogLevels()

#Description:
#   Set the python log level to the given level
#Input Arguments:
#   Log level to set
#Return:
#   None
#SetServerLogLevel
handle.SetServerLogLevel(PythonLogLevels["DEBUG"])

#Description:
#   Set the metal log level to the given level
#Input Arguments:
#   Log level to set
#Return:
#   None
#SetMetalLogLevel
metal_log_level = handle.GetEnum_metal_log_level()
handle.SetMetalLogLevel(metal_log_level["METAL_LOG_EMERGENCY"])

#Description:
#   Return Dictionary with all CCF macros in the ccf header file
#Input Arguments:
#   None
#Return:
#   Dictionary with all CCF macros in the ccf header file
#GetCcfMacro
ccf_macro = handle.GetCcfMacro()
print(ccf_macro["XDFECCF_MAX_NUM_INSTANCES"])
print(ccf_macro["XST_SUCCESS"])
print(ccf_macro["XST_FAILURE"])
print(ccf_macro["XDFECCF_NODE_NAME_MAX_LENGTH"])
print(ccf_macro["XDFECCF_CC_NUM"])
print(ccf_macro["XDFECCF_ANT_NUM_MAX"])
print(ccf_macro["XDFECCF_SEQ_LENGTH_MAX"])
print(ccf_macro["XDFECCF_NUM_COEFF"])

#Description:
#   Return Dictionary equivalent of enum XDfeCcf_StateId
#Input Arguments:
#   None
#Return:
#   Dictionary equivalent of enum XDfeCcf_StateId
#GetEnum_XDfeCcf_StateId
XDfeCcf_StateId = handle.GetEnum_XDfeCcf_StateId()

#Description:
#   Return Dictionary equivalent of enum metal_log_level
#Input Arguments:
#   None
#Return:
#   Dictionary equivalent of enum metal_log_level
#GetEnum_metal_log_level
metal_log_level = handle.GetEnum_metal_log_level()

#Description:
#   Return Dictionary equivalent of structure XDfeCcf_InternalCarrierCfg
#Input Arguments:
#   None
#Return:
#   Dictionary equivalent of structure XDfeCcf_InternalCarrierCfg
#GetStruct_XDfeCcf_InternalCarrierCfg
XDfeCcf_InternalCarrierCfg = handle.GetStruct_XDfeCcf_InternalCarrierCfg()

#Description:
#   Initialises one instance of a channel filter driver.
#   Traverses "/sys/bus/platform/device" directory (in Linux), to find registered
#   CCF device with the name DeviceNodeName. The first available slot in
#   the instances array XDfeCcf_ChFilter[] will be taken as a DeviceNodeName
#   object. On success it moves the state machine to a Ready state, while on
#   failure stays in a Not Ready state.
#C header declaration:
#   XDfeCcf *XDfeCcf_InstanceInit(const char *DeviceNodeName);
#Input Arguments:
#   DeviceNodeName: device node name.
#Return:
#   ret - 0 on success, 1 on failure
#   device_id - integer handle to the initialized instance
#   DeviceNodeNameRet - device node name returned from driver
#                       which will be same as the passed value
#Notes:
#   The bytes conversion is already done inside xpyro client.

DeviceNodeName = "a7c00000.xdfe_cc_filter"

ret, device_id, DeviceNodeName = handle.XDfeCcf_InstanceInit(DeviceNodeName)

#Description:
#   Writes value to register in a Ccf instance.
#C header declaration:
#   void XDfeCcf_WriteReg(const XDfeCcf *InstancePtr, u32 AddrOffset, u32 Data);
#Input Arguments:
#   device_id: id of the opened device.
#   AddrOffset: address offset relative to instance base address.
#   Data: value to be written.
#Return:
#   None

handle.XDfeCcf_WriteReg(device_id, 0x40, 0)

#Description:
#   Reads a value from register using a Ccf instance.
#C header declaration:
#   u32 XDfeCcf_ReadReg(const XDfeCcf *InstancePtr, u32 AddrOffset);
#Input Arguments:
#   device_id: id of the opened device.
#   AddrOffset: address offset relative to instance base address
#Return:
#   regval: Register value.

ret = handle.XDfeCcf_ReadReg(device_id, 0)

#Description:
#   Resets channel filter and puts block into a reset state.
#C header declaration:
#   void XDfeCcf_Reset(XDfeCcf *InstancePtr);
#Input Arguments:
#   device_id: id of the opened device.
#Return:
#   None

handle.XDfeCcf_Reset(device_id)

#Description:
#   Read configuration from device tree/xparameters.h and IP registers.
#   Removes S/W reset and moves the state machine to a Configured state.
#C header declaration:
#   void XDfeCcf_Configure(XDfeCcf *InstancePtr, XDfeCcf_Cfg *Cfg);
#Input Arguments:
#   device_id: id of the opened device.
#   Cfg: configuration data container
#Return:
#   Cfg: configuration data container

Cfg_in = handle.GetStruct_XDfeCcf_Cfg()
Cfg_out = handle.XDfeCcf_Configure(device_id, Cfg_in)

#Description:
#   DFE Ccf driver one time initialisation, also moves the state machine to
#   an Initialised state.
#C header declaration:
#   void XDfeCcf_Initialize(XDfeCcf *InstancePtr, const XDfeCcf_Init *Init);
#Input Arguments:
#   device_id: id of the opened device.
#   Init: initialisation data container
#Return:
#   Init: initialisation data container

Init_in = handle.GetStruct_XDfeCcf_Init()
Init_in['Sequence']['Length']=6
Init_out = handle.XDfeCcf_Initialize(device_id, Init_in)

#Description:
#   Set trigger configuration.
#C header declaration:
#   void XDfeCcf_SetTriggersCfg(const XDfeCcf *InstancePtr, XDfeCcf_TriggerCfg *TriggerCfg);
#Input Arguments:
#   device_id: id of the opened device.
#   TriggerCfg: is a trigger configuration container.
#Return:
#   TriggerCfg: is a trigger configuration container.

TriggerCfg_in = handle.GetStruct_XDfeCcf_TriggerCfg()
TriggerCfg_in['Activate']['TriggerEnable'] = 0
TriggerCfg_in['Activate']['Mode'] = 0
TriggerCfg_in['Activate']['TuserEdgeLevel'] = 0
TriggerCfg_in['Activate']['StateOutput'] = 0
TriggerCfg_in['Activate']['TUSERBit'] = 0
TriggerCfg_in['CCUpdate']['TriggerEnable'] = 0
TriggerCfg_in['CCUpdate']['Mode'] = 0
TriggerCfg_in['CCUpdate']['TuserEdgeLevel'] = 0
TriggerCfg_in['CCUpdate']['StateOutput'] = 0
TriggerCfg_in['CCUpdate']['TUSERBit'] = 0
TriggerCfg_out = handle.XDfeCcf_SetTriggersCfg(device_id, TriggerCfg_in)

#Description:
#   Activates channel filter and moves the state machine to an Activated state.
#C header declaration:
#   void XDfeCcf_Activate(XDfeCcf *InstancePtr, bool EnableLowPower);
#Input Arguments:
#   device_id: id of the opened device.
#   EnableLowPower: flag indicating low power
#Return:
#   None

handle.XDfeCcf_Activate(device_id, 0)

#Description:
#   Gets a state machine state id. The state id is returned as a string.
#   The enum is mapped to a dictionary in python.
#C header declaration:
#   XDfeCcf_StateId XDfeCcf_GetStateID(XDfeCcf *InstancePtr);
#Input Arguments:
#   device_id: id of the opened device.
#Return:
#   StateId: State machine StateID as a string

StateId = handle.XDfeCcf_GetStateID(device_id)
print(StateId)

#Description:
#   Returns the current CC configuration. Not used slot ID in a sequence
#   (Sequence.CCID[Index]) are represented as (-1), not the value in registers.
#C header declaration:
#   void XDfeCcf_GetCurrentCCCfg(const XDfeCcf *InstancePtr, XDfeCcf_CCCfg *CCCfg);
#Input Arguments:
#   device_id: id of the opened device.
#   CurrCCCfg: CC configuration container.
#Return:
#   CurrCCCfg: CC configuration container
#Note:
#   For a sequence conversion see XDfeCcf_AddCCtoCCCfg() comment.

CurrCCCfg = handle.GetStruct_XDfeCcf_CCCfg()
CurrentCCCfg = handle.XDfeCcf_GetCurrentCCCfg(device_id, CurrCCCfg)

#Description:
#   Returns the current CC configuration for DL and UL in switchable mode.
#   Not used slot ID in a sequence (Sequence.CCID[Index]) are represented
#   as (-1), not the value in registers.
#C header declaration:
#   void XDfeCcf_GetCurrentCCCfgSwitchable(const XDfeCcf *InstancePtr,
#			     XDfeCcf_CCCfg *CCCfgDownlink,
#			     XDfeCcf_CCCfg *CCCfgUplink);
#Input Arguments:
#   device_id: id of the opened device.
#   CCCfgDownlink: Downlink CC configuration container.
#   CCCfgUplink: Uplink CC configuration container.
#Return:
#   CCCfgDownlink, CCCfgUplink: CC configuration container

if (switchable_flag == True):
	CCCfgDownlink = handle.GetStruct_XDfeCcf_CCCfg()
	CCCfgUplink = handle.GetStruct_XDfeCcf_CCCfg()
	CCCfgDownlink, CCCfgUplink = handle.XDfeCcf_GetCurrentCCCfgSwitchable(device_id, CCCfgDownlink, CCCfgUplink)

#Description:
#   Returns configuration structure CCCfg with CCCfg->Sequence.Length value set
#   in XDfeCcf_Configure(), array CCCfg->Sequence.CCID[] members are set to not
#   used value (-1) and the other CCCfg members are set to 0.
#C header declaration:
#   void XDfeCcf_GetEmptyCCCfg(const XDfeCcf *InstancePtr, XDfeCcf_CCCfg *CCCfg);
#Input Arguments:
#   device_id: id of the opened device.
#Return:
#   CCCfg: CC configuration container

CCCfg = handle.XDfeCcf_GetEmptyCCCfg(device_id)

#Description:
#   Returns the current CCID carrier configuration.
#C header declaration:
#   void XDfeCcf_GetCarrierCfg(const XDfeCcf *InstancePtr, XDfeCcf_CCCfg *CCCfg,
#	s32 CCID, u32 *CCSeqBitmap, XDfeCcf_CarrierCfg *CarrierCfg);
#Input Arguments:
#   device_id: id of the opened device.
#   CCCfg: component carrier (CC) configuration container.
#   CCID: Channel ID.
#Return:
#   CCSeqBitmap: CC slot position container.
#   CarrierCfg: CC configuration container.

CCCfg = handle.GetStruct_XDfeCcf_CCCfg()
CCSeqBitmap, CarrierCfg = handle.XDfeCcf_GetCarrierCfg(device_id, CCCfg, 0)

#Description:
#   Set antenna configuration in CC configuration container.
#C header declaration:
#   void XDfeCcf_SetAntennaCfgInCCCfg(const XDfeCcf *InstancePtr,
#   XDfeCcf_CCCfg *CCCfg, u32 *AntennaCfg);
#Input Arguments:
#   device_id: id of the opened device.
#   CCCfg: component carrier (CC) configuration container.
#   AntennaCfg: Array of all antenna configurations.
#Return:
#   CCCfg: component carrier (CC) configuration container.

AntennaCfg = handle.GetStruct_XDfeCcf_AntennaCfg()
AntennaCfg = {
	"Enable": [1,1,1,1,1,1,0,0],
}
CCCfg = handle.XDfeCcf_SetAntennaCfgInCCCfg(device_id, CCCfg, AntennaCfg)

#Description:
#   Updates antenna configuration to all antennas.
#C header declaration:
#   u32 XDfeCcf_UpdateAntennaCfg(XDfeCcf *InstancePtr, XDfeCcf_AntennaCfg *AntennaCfg);
#Input Arguments:
#   device_id: id of the opened device.
#   AntennaCfg: Array of all antenna configurations.
#Return:
#   ret - XST_SUCCESS if successful, XST_FAILURE if error occurs.
#Note:
#   Clear event status with XDfeCcf_ClearEventStatus() before running this API.

AntennaCfg = handle.GetStruct_XDfeCcf_AntennaCfg()
AntennaCfg = {
	"Enable": [1,1,1,1,1,1,0,0],
}
ret = handle.XDfeCcf_UpdateAntennaCfg(device_id, AntennaCfg)

#Description:
#   Updates antenna configuration of all antennas. Applies gain to downlink only
#   in switchable mode.
#C header declaration:
#   u32 XDfeCcf_UpdateAntennaCfgSwitchable(XDfeCcf *InstancePtr,
#				       XDfeCcf_AntennaCfg *AntennaCfgDownlink,
#				       XDfeCcf_AntennaCfg *AntennaCfgUplink)
#Input Arguments:
#   device_id: id of the opened device.
#   AntennaCfgDownlink: Array of all downlink antenna configurations.
#   AntennaCfgUplink: Array of all uplink antenna configurations.
#Return:
#   ret - XST_SUCCESS if successful, XST_FAILURE if error occurs.
#Note:
#   Clear event status with XDfeCcf_ClearEventStatus() before running this API.

if (switchable_flag == True):
	AntennaCfgDL = handle.GetStruct_XDfeCcf_AntennaCfg()
	AntennaCfgDL = {"Enable": [1,1,1,1,1,1,0,0]}
	AntennaCfgUL = handle.GetStruct_XDfeCcf_AntennaCfg()
	AntennaCfgUL = {"Enable": [1,1,1,1,1,1,0,0]}
	ret = handle.XDfeCcf_UpdateAntennaCfgSwitchable(device_id, AntennaCfgDL, AntennaCfgUL)

#Description:
#   Adds specified CCID, with specified configuration, to a local CC
#   configuration structure.
#   If there is insufficient capacity for the new CC the function will return
#   an error.
#   Initiates CC update (enable CCUpdate trigger TUSER Single Shot).
#
#   The returned CCCfg.Sequence is transleted as there is no explicite indication that
#   SEQUENCE[i] is not used - 0 can define the slot as either used or not used.
#   Sequence data that is returned in the CCIDSequence is not the same as what is
#   written in the registers. The translation is:
#       - CCIDSequence.CCID[i] = -1    - if [i] is unused slot
#       - CCIDSequence.CCID[i] = CCID  - if [i] is used slot
#       - a returned CCIDSequence->Length = length in register + 1
#C header declaration:
#   u32 XDfeCcf_AddCCtoCCCfg(XDfeCcf *InstancePtr, XDfeCcf_CCCfg *CCCfg, s32 CCID,
#	u32 CCSeqBitmap, const XDfeCcf_CarrierCfg *CarrierCfg);
#Input Arguments:
#   device_id: id of the opened device.
#   CCCfg: component carrier (CC) configuration container.
#   CCID: Channel ID.
#   CCSeqBitmap: CC slot position container.
#   CarrierCfg: CC configuration container.
#Return:
#   ret: XST_SUCCESS if successful, XST_FAILURE if error occurs.
#   CCCfg: component carrier (CC) configuration container.

CCCfg = handle.GetStruct_XDfeCcf_CCCfg()
CCCfg = handle.XDfeCcf_GetCurrentCCCfg(device_id, CCCfg)
CCID = 0x1
CCSeqBitmap = 0x2
CarrierCfg = handle.GetStruct_XDfeCcf_CarrierCfg()
Status = handle.GetStruct_XDfeCcf_Status()
Status['CCUpdate'] = 1
handle.XDfeCcf_ClearEventStatus(int(device_id), Status)
ret, CCCfg = handle.XDfeCcf_AddCCtoCCCfg(int(device_id), CCCfg, CCID, CCSeqBitmap, CarrierCfg)

#Description:
#   Removes specified CCID from a local CC configuration structure.
#C header declaration:
#   void XDfeCcf_RemoveCCfromCCCfg(XDfeCcf *InstancePtr, XDfeCcf_CCCfg *CCCfg, s32 CCID);
#Input Arguments:
#   device_id: id of the opened device.
#   CCCfg: component carrier (CC) configuration container.
#   CCID: Channel ID.
#Return:
#   CCCfg: component carrier (CC) configuration container.
#Note:
#   For a sequence conversion see XDfeCcf_AddCCtoCCCfg comment.

CCCfg = handle.GetStruct_XDfeCcf_CCCfg()
CCCfg = handle.XDfeCcf_RemoveCCfromCCCfg(device_id, CCCfg, 0)

#Description:
#   Updates specified CCID, with specified configuration to a local CC
#   configuration structure.
#   If there is insufficient capacity for the new CC the function will return
#   an error.
#C header declaration:
#   void XDfeCcf_UpdateCCinCCCfg(const XDfeCcf *InstancePtr, XDfeCcf_CCCfg *CCCfg,
# 	s32 CCID, const XDfeCcf_CarrierCfg *CarrierCfg);
#Input Arguments:
#   device_id: id of the opened device.
#   CCCfg: component carrier (CC) configuration container.
#   CCID: Channel ID.
#   CarrierCfg: CC configuration container.
#Return:
#   CCCfg: component carrier (CC) configuration container.

CCCfg = handle.GetStruct_XDfeCcf_CCCfg()
CarrierCfg = handle.GetStruct_XDfeCcf_CarrierCfg()
CCCfg = handle.XDfeCcf_UpdateCCinCCCfg(device_id, CCCfg, 0, CarrierCfg)

#Description:
#   Writes local CC configuration to the shadow (NEXT) registers and triggers
#   copying from shadow to operational registers.
#C header declaration:
#   u32 XDfeCcf_SetNextCCCfgAndTrigger(const XDfeCcf *InstancePtr, XDfeCcf_CCCfg *CCCfg);
#Input Arguments:
#   device_id: id of the opened device.
#   CCCfg: component carrier (CC) configuration container.
#Return:
#   ret: XST_SUCCESS if successful, XST_FAILURE if error occurs.
#   CCCfg: component carrier (CC) configuration container.

CCCfg = handle.GetStruct_XDfeCcf_CCCfg()
ret, CCCfg = handle.XDfeCcf_SetNextCCCfgAndTrigger(device_id, CCCfg)

#Description:
#   Writes local CC configuration to the shadow (NEXT) registers and triggers
#   copying from shadow to operational (CURRENT) registers.
#C header declaration:
#   u32 XDfeCcf_SetNextCCCfgAndTriggerSwitchable(const XDfeCcf *InstancePtr,
#					     XDfeCcf_CCCfg *CCCfgDownlink,
#					     XDfeCcf_CCCfg *CCCfgUplink)
#Input Arguments:
#   device_id: id of the opened device.
#   CCCfgDownlink: Downlink CC configuration container.
#   CCCfgUplink: Uplink CC configuration container.
#Return:
#   ret: XST_SUCCESS if successful, XST_FAILURE if error occurs.
#   CCCfgDownlink: Downlink CC configuration container.
#   CCCfgUplink: Uplink CC configuration container.

if (switchable_flag == True):
	CCCfgDownlink = handle.GetStruct_XDfeCcf_CCCfg()
	CCCfgUplink = handle.GetStruct_XDfeCcf_CCCfg()
	ret, CCCfgDownlink, CCCfgUplink = handle.XDfeCcf_SetNextCCCfgAndTriggerSwitchable(device_id, CCCfgDownlink, CCCfgUplink)

#Description:
#   Adds specified CCID, with specified configuration.
#   If there is insufficient capacity for the new CC the function will return
#   an error.
#   Initiates CC update (enable CCUpdate trigger TUSER Single Shot).
#C header declaration:
#   u32 XDfeCcf_AddCC(const XDfeCcf *InstancePtr, u32 CCID,
#   const XDfeCcf_CarrierCfg *CarrierCfg);
#Input Arguments:
#   device_id: id of the opened device.
#   CCID: Channel ID.
#   CCSeqBitmap: up to 16 defined slots into which a CC can be
#                allocated. The number of slots can be from 1 to 16 depending on
#                system initialization. The number of slots is defined by the
#                "sequence length" parameter which is provided during initialization.
#                The Bit offset within the CCSeqBitmap indicates the equivalent
#                Slot number to allocate. e.g. 0x0003  means the caller wants the
#                passed component carrier (CC) to be allocated to slots 0 and 1.
#   CarrierCfg: is a CC configuration container.
#Return:
#   XST_SUCCESS if successful, XST_FAILURE if error occurs.
#   CarrierCfg: is a CC configuration container.

CarrierCfg_in = handle.GetStruct_XDfeCcf_CarrierCfg()
CCSeqBitmap = 0xb
ret, CarrierCfg_out = handle.XDfeCcf_AddCC(device_id, 0, CCSeqBitmap, CarrierCfg_in)

#Description:
#   Removes specified CCID.
#   Initiates CC update (enable CCUpdate trigger TUSER Single Shot).
#C header declaration:
#   u32 XDfeCcf_RemoveCC(const XDfeCcf *InstancePtr, u32 CCID);
#Input Arguments:
#   device_id: id of the opened device.
#   CCID: is a Channel ID.
#Return:
#   XST_SUCCESS if successful, XST_FAILURE if error occurs.
#Note:
#   Clear event status with XDfeCcf_ClearEventStatus() before running this API.

ret = handle.XDfeCcf_RemoveCC(device_id, 0)

#Description:
#   Updates specified CCID carrier configuration; change gain or filter
#   coefficients set.
#   Initiates CC update (enable CCUpdate trigger TUSER Single Shot).
#C header declaration:
#   u32 XDfeCcf_UpdateCC(const XDfeCcf *InstancePtr, u32 CCID,
#   XDfeCcf_CarrierCfg *CarrierCfg);
#Input Arguments:
#   device_id: id of the opened device.
#   CCID: is a Channel ID.
#   CarrierCfg: is a CC configuration container.
#Return:
#   ret - XST_SUCCESS if successful, XST_FAILURE if error occurs.
#   CarrierCfg - CC configuration container

CarrierCfg_in = handle.GetStruct_XDfeCcf_CarrierCfg()
ret, CarrierCfg_out = handle.XDfeCcf_UpdateCC(device_id, 0, CarrierCfg_in)

#Description:
#   Updates specified antenna TDM slot enablement.
#   Initiates CC update (enable CCUpdate trigger one-shot).
#C header declaration:
#   u32 XDfeCcf_UpdateAntenna(const XDfeCcf *InstancePtr, u32 Ant, bool Enabled);
#Input Arguments:
#   device_id: id of the opened device.
#   Ant: is antenna ID.
#   Enabled: flag indicating enable status of the antenna.
#Return:
#   ret - XST_SUCCESS if successful, XST_FAILURE if error occurs.
#Note:
#   Clear event status with XDfeCcf_ClearEventStatus() before running this API.

ret = handle.XDfeCcf_UpdateAntenna(device_id, 0, 0)

#Description:
#   Return current trigger configuration.
#C header declaration:
#   void XDfeCcf_GetTriggersCfg(const XDfeCcf *InstancePtr,
#	XDfeCcf_TriggerCfg *TriggerCfg);
#Input Arguments:
#   device_id: id of the opened device.
#Return:
#   TriggerCfg: is a trigger configuration container.

TriggerCfg_out = handle.XDfeCcf_GetTriggersCfg(device_id)

#Description:
#   Get specified CCID carrier configuration.
#C header declaration:
#   void XDfeCcf_GetCC(const XDfeCcf *InstancePtr, u32 CCID,
#	XDfeCcf_CarrierCfg *CarrierCfg);
#Input Arguments:
#   device_id: id of the opened device.
#   CCID: is a Channel ID.
#Return:
#   CarrierCfg: is a trigger configuration container.

CarrierCfg_out = handle.XDfeCcf_GetCC(device_id, 0)

#Description:
#   Return a list indicating which coefficient sets are currently in use.
#C header declaration:
#   void XDfeCcf_GetActiveSets(const XDfeCcf *InstancePtr, u32 *IsActive);
#Input Arguments:
#   device_id: id of the opened device.
#Return:
#   IsActive: variable indicating an activation status.

IsActive = handle.XDfeCcf_GetActiveSets(device_id)

#Description:
#   Writes the coefficient set defined into the register map and commit them
#   to the hard block's internal coefficient memory for the specified Set.
#C header declaration:
#   void XDfeCcf_LoadCoefficients(const XDfeCcf *InstancePtr, u32 Set, u32 Shift,
#   const XDfeCcf_Coefficients *Coeffs);
#Input Arguments:
#   device_id: id of the opened device.
#   Set: coefficient set Id
#   Shift: is a coefficient shift value.
#   Coeffs: an array of filter coefficients
#Return:
#   Coeffs: an array of filter coefficients

#Coeffs_in = handle.GetStruct_XDfeCcf_Coeffs()
Coeffs_in = {
    "Num": 7,
    "Symmetric": 1,
    "Value": [0, 0, 0, (2 ** 15) - 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
}
Coeffs_out = handle.XDfeCcf_LoadCoefficients(device_id, 0, 8, Coeffs_in)

#Description:
#   Get overflow event status
#C header declaration:
#   void XDfeCcf_GetOverflowStatus(const XDfeCcf *InstancePtr,
#			       XDfeCcf_OverflowStatus *Status);
#Input Arguments:
#   device_id: id of the opened device.
#Return:
#   Status: overflow event status
OverflowStatus = handle.XDfeCcf_GetOverflowStatus(device_id)

#Description:
#   Sets uplink/downlink register bank.
#C header declaration:
#   void XDfeCcf_SetRegBank(const XDfeCcf *InstancePtr,
#                  u32 RegBank)
#Input Arguments:
#   device_id: id of the opened device.
#   RegBank: Register bank value to be set.
#Return:
#   None

if (switchable_flag == True):
	RegBank = 1
	handle.XDfeCcf_SetRegBank(device_id, RegBank)
	RegBank = 0
	handle.XDfeCcf_SetRegBank(device_id, RegBank)

#Description:
#   Gets event status.
#C header declaration:
#   void XDfeCcf_GetEventStatus(const XDfeCcf *InstancePtr, XDfeCcf_Status *Status);
#Input Arguments:
#   device_id: id of the opened device.
#Return:
#   Status: event status

Status = handle.XDfeCcf_GetEventStatus(device_id)

#Description:
#   Clears event status.XDfeCcf_ClearEventStatus
#C header declaration:
#   void XDfeCcf_ClearEventStatus(const XDfeCcf *InstancePtr);
#Input Arguments:
#   device_id: id of the opened device.
#   Status: event status.
#       0 - does not clear corresponding event status
#       1 - clears corresponding event status
#Return:
#   None

Status_in = handle.GetStruct_XDfeCcf_Status()
handle.XDfeCcf_ClearEventStatus(device_id, Status_in)

#Description:
#   Sets interrupt masks.
#C header declaration:
#   void XDfeCcf_SetInterruptMask(const XDfeCcf *InstancePtr,
#	const XDfeCcf_InterruptMask *Mask);
#Input Arguments:
#   device_id: id of the opened device.
#   Mask: interrupt mask value
#       0 - does not mask corresponding interrupt
#       1 - masks corresponding interrupt
#Return:
#   None

Mask_in = handle.GetStruct_XDfeCcf_InterruptMask()
handle.XDfeCcf_SetInterruptMask(device_id, Mask_in)

#Description:
#   Get Interrupt mask
#C header declaration:
#   void XDfeCcf_GetInterruptMask(const XDfeCcf *InstancePtr,
#	const XDfeCcf_InterruptMask *Mask);
#Input Arguments:
#   device_id: id of the opened device.
#Return:
#   Mask: interrupt mask value.

Mask_out = handle.XDfeCcf_GetInterruptMask(device_id)

#Description:
#   Reads the delay, which will be added to TUSER and TLAST (delay matched
#   through the IP).
#C header declaration:
#   u32 XDfeCcf_GetTUserDelay(const XDfeCcf *InstancePtr);
#Input Arguments:
#   device_id: id of the opened device.
#Return:
#   ret: Delay value

ret = handle.XDfeCcf_GetTUserDelay(device_id)
print (ret)

#Description:
#   Gets calculated TDataDelay value for CCID from current CC configuration.
#C header declaration:
#   u32 XDfeCcf_GetTDataDelay(const XDfeCcf *InstancePtr, u32 Tap);
#Input Arguments:
#   device_id: id of the opened device.
#   Tap: is a tap variable.
#   CCID: CC ID.
#   Symmetric: Select symetric (1) or non-symetric (0) filter.
#   Num: Number of coefficients values.
#Return:
#   ret: XST_SUCCESS if successful, XST_FAILURE if error occurs.
#   TDataDelay: Delay value container.

CCID = 0x0
CCSeqBitmap = 0x1
CarrierCfg_in = handle.GetStruct_XDfeCcf_CarrierCfg()
CarrierCfg_in['Gain']=0
CarrierCfg_in['ImagCoeffSet']=0
CarrierCfg_in['RealCoeffSet']=0
Status = handle.GetStruct_XDfeCcf_Status()
Status['CCUpdate'] = 1
handle.XDfeCcf_ClearEventStatus(device_id, Status)
CarrierCfg_out = handle.XDfeCcf_AddCC(device_id, CCID, CCSeqBitmap, CarrierCfg_in)
print(CarrierCfg_out)
CCCfg = handle.GetStruct_XDfeCcf_CCCfg()
CCCfg = handle.XDfeCcf_GetCurrentCCCfg(device_id, CCCfg)
Tap = 0
Symmetric = 1
Num = 26
ret, DataLatency = handle.XDfeCcf_GetTDataDelay(device_id, Tap, CCID, Symmetric, Num)
print (DataLatency)

#Description:
#   Gets calculated TDataDelay value for CCID.
#C header declaration:
#   u32 XDfeCcf_GetTDataDelayFromCCCfg(XDfeCcf *InstancePtr, u32 Tap, s32 CCID,
#   XDfeCcf_CCCfg *CCCfg);
#Input Arguments:
#   device_id: id of the opened device.
#   Tap: is a tap variable.
#   CCID: CC ID.
#   CCCfg: Component carrier (CC) configuration container.
#   Symmetric: Select symetric (1) or non-symetric (0) filter.
#   Num: Number of coefficients values.
#Return:
#   ret: XST_SUCCESS if successful, XST_FAILURE if error occurs.
#   TDataDelay: Delay value container.

CurrCCCfg = handle.GetStruct_XDfeCcf_CCCfg()
CurrentCCCfg = handle.XDfeCcf_GetCurrentCCCfg(device_id, CurrCCCfg)
ret, DataLatency = handle.XDfeCcf_GetTDataDelayFromCCCfg(device_id,  Tap, CCID, CurrentCCCfg, Symmetric, Num)
print (DataLatency)

#Description:
#   This API is used to get the driver version.
#C header declaration:
#   void XDfeCcf_GetVersions(const XDfeCcf *InstancePtr, XDfeCcf_Version *SwVersion,
#	XDfeCcf_Version *HwVersion);
#Input Arguments:
#   device_id: id of the opened device.
#Return:
#   SwVersion: driver version numbers.
#   HwVersion: HW version numbers.

SwVersion_out, HwVersion_out = handle.XDfeCcf_GetVersions(device_id)

#Description:
#   Deactivates channel filter and moves the state machine to Initialised state.
#C header declaration:
#   void XDfeCcf_Deactivate(XDfeCcf *InstancePtr);
#Input Arguments:
#   device_id: id of the opened device.
#Return:
#   None

handle.XDfeCcf_Deactivate(device_id)

#Description:
#   Sets the delay, which will be added to TUSER and TLAST (delay matched
#   through the IP).
#C header declaration:
#   u32 XDfeCcf_SetTUserDelay(const XDfeCcf *InstancePtr, u32 Delay);
#Input Arguments:
#   device_id: id of the opened device.
#   Delay: requested delay variable.
#Return:
#   None

handle.XDfeCcf_SetTUserDelay(device_id, 10)

#Description:
#   Closes the instances of a channel filter driver and moves the state
#   machine to a Not Ready state.
#C header declaration:
#   void XDfeCcf_InstanceClose(XDfeCcf *InstancePtr);
#Input Arguments:
#   device_id: id of the opened device.
#Return:
#   None

handle.XDfeCcf_InstanceClose(device_id)



# Copyright (C) 2024-2025 Advanced Micro Devices, Inc.  All rights reserved.
# SPDX-License-Identifier: BSD-3-Clause

__author__ = "Salih Erim"
__copyright__ = "Copyright 2024-2025, Advanced Micro Devices, Inc."

from enum import IntEnum
from periphery import I2C, GPIO
import threading

PM_LOWLEVEL_DEBUG = False

class PMBUS(IntEnum):
    PAGE =          0x00
    OPERATION =     0x01
    ON_OFF_CONFIG = 0x02
    CLEAR_FAULTS =  0x03
    PHASE =         0x04
    VOUT_MODE =     0x20
    VOUT_COMMAND =  0x21
    VOUT_MAX =      0X24
    VOUT_MIN =      0x2B
    VOUT_SCALE_LOOP = 0x29
    STATUS_BYTE =   0x79
    STATUS_WORD =   0x79
    STATUS_VOUT =   0x7A
    STATUS_IOUT =   0x7B
    STATUS_INPUT =  0x7C
    STATUS_TEMP =   0x7D
    STATUS_CML =    0x7E
    STATUS_OTHER =  0x7F
    READ_VOUT =     0x8B
    READ_IOUT =     0x8C
    READ_TEMP_1 =   0x8D
    IC_DEVICE_ID =  0xAD
    IC_DEVICE_REV = 0xAE

class ScalingType(IntEnum):
    VID = 1
    LINEAR16 = 2
    LINEAR11 = 3
    DIRECT = 4

def pm_print(printstr):
    if(PM_LOWLEVEL_DEBUG):
        print(printstr)

class PMBusRegulator:
    vout_scaling = ScalingType.LINEAR16
    iout_scaling = ScalingType.LINEAR11
    temp_scaling = None
    def __init__(self, device_name, device_path, device_address, page=-1, pmbus_vout_mode=-1, phase=-1, alert_gpio_pin=None):
        """
        Initialize a PMBus device with a specific page (output).

        :param bus_number: I2C bus number (e.g., 1 for /dev/i2c-1)
        :param device_address: I2C address of the PMIC
        :param page: PMBus page number corresponding to the output
        :param alert_gpio_pin: Optional GPIO pin for PMBus ALERT (None if not used)
        """
        self.name = device_name
        self.i2c = I2C(device_path)
        self.addr = int(device_address, 0)
        self.page = page
        self.pmbus_vout_mode = pmbus_vout_mode
        self.phase = phase
        self.vout_mode = 0

        match self.name:
            case 'TPS53681':
                self.vout_scaling = ScalingType.VID
                self.temp_scaling = ScalingType.LINEAR11
            case 'MPQ2283' | 'MPQ2285' | 'MPQ72963':
                self.vout_scaling = ScalingType.DIRECT
                self.iout_scaling = None
                self.temp_scaling = None
            case _:
                self.vout_scaling = ScalingType.LINEAR16

        if self.pmbus_vout_mode <= 0:
            self.vout_mode = 0x18 # 0x18 - 0x20 = -8

        # Initialize ALERT pin if provided
        self.alert_gpio = GPIO(alert_gpio_pin, "in") if alert_gpio_pin is not None else None
        if self.alert_gpio:
            self.alert_gpio.edge = "falling"
            self.alert_thread = threading.Thread(target=self._alert_monitor)
            self.alert_thread.daemon = True
            self.alert_thread.start()

    def __str__(self):
        str_info = {
            k: v
            for k, v in self.__dict__.items()
            if k not in ['i2c'] and v
        }
        return str(str_info)

    def close(self):
        """Clean up resources."""
        self.i2c.close()
        if self.alert_gpio:
            self.alert_gpio.close()

    def enable_output(self):
        """Enable output via OPERATION command."""
        if self.page >= 0:
            self._select_page()
        config = self._read_byte(PMBUS.ON_OFF_CONFIG)
        if config:
            config |= 0x08
            self._write_byte(PMBUS.ON_OFF_CONFIG, config)
            pm_print("ON_OFF_CONFIG = 0x{0:02x}".format(config))
        self._write_byte(PMBUS.OPERATION, 0x80)

    def shutdown_output(self):
        """Disable output via OPERATION command."""
        if self.page >= 0:
            self._select_page()
        config = self._read_byte(PMBUS.ON_OFF_CONFIG)
        if config:
            config |= 0x08
            self._write_byte(PMBUS.ON_OFF_CONFIG, config)
            pm_print("ON_OFF_CONFIG = 0x{0:02x}".format(config))
        self._write_byte(PMBUS.OPERATION, 0x00)

    def set_voltage(self, value):
        pm_print(f"set_voltage({value})")
        if self.name in ("MPQ2283", "MPQ2285"):
            raw_value = self._value_2_rawvalue(value, self.vout_scaling)
            pm_print("raw_value 0x{0:02x}".format(raw_value))
            self._write_byte(PMBUS.VOUT_COMMAND, raw_value)
        else:
            #self.shutdown_output()
            if self.page >= 0:
                self._select_page()
            self._get_vout_mode()
            raw_value = self._value_2_rawvalue(value, self.vout_scaling)
            self._write_word(PMBUS.VOUT_COMMAND, raw_value)
            pm_print("raw_value 0x{0:04x}".format(raw_value))
            #self.enable_output()

    def read_voltage(self):
        """Read and scale output voltage using READ_VOUT command."""
        pm_print(f"read_voltage()")
        if not self.vout_scaling:
            return None
        if self.page >= 0:
            self._select_page()
        self._get_vout_mode()
        if self.name in ("MPQ2283", "MPQ2285"):
            raw_voltage = self._read_byte(PMBUS.VOUT_COMMAND)
        else:
            raw_voltage = self._read_word(PMBUS.READ_VOUT)
        if not raw_voltage:
            return None
        else:
            pm_print("raw_voltage 0x{0:04x}".format(raw_voltage))
            return round(self._rawvalue_2_value(raw_voltage, self.vout_scaling), 4)

    def read_current(self):
        """Read and scale output current using READ_IOUT command."""
        if not self.iout_scaling:
            return None
        if self.page >= 0:
            self._select_page()
        if self.phase > 0:
            self._write_byte(PMBUS.PHASE, 0x80) # 0x80 to read whole phases as total
        raw_current = self._read_word(PMBUS.READ_IOUT)
        if not raw_current:
            return None
        else:
            pm_print("raw_current 0x{0:04x}".format(raw_current))
            return self._rawvalue_2_value(raw_current, self.iout_scaling)

    def read_temperature(self):
        """Read and scale output temperature using READ_TEMPERATURE_1 command."""
        if not self.temp_scaling:
            return None
        if self.page >= 0:
            self._select_page()
        raw_temp = self._read_word(PMBUS.READ_TEMP_1)
        if not raw_temp:
            return None
        else:
            pm_print("raw_temp 0x{0:04x}".format(raw_temp))
            return self._rawvalue_2_value(raw_temp, self.temp_scaling)

    def read_vin(self):
        return None

    def read_iin(self):
        return None

    def read_pout(self):
        return None

    def read_pin(self):
        return None

    def read_telemetry_all(self):
        telemetry = []
        telemetry.append(self.read_vin())
        telemetry.append(self.read_iin())
        telemetry.append(self.read_voltage())
        telemetry.append(self.read_current())
        telemetry.append(self.read_temperature())
        telemetry.append(self.read_pout())
        telemetry.append(self.read_pin())
        return telemetry

    def clear_faults(self):
        """Clear all recorded faults."""
        self._write_byte(PMBUS.CLEAR_FAULTS, 0x00)

    def _linear11_to_float(self, linear11):
        """ Convert a Linear11 formatted value to a float. """
        exponent = (linear11 >> 11) & 0x1F
        if exponent & 0x10:
            exponent -= 0x20
        mantissa = linear11 & 0x07FF
        if mantissa & 0x0400:
            mantissa -= 0x0800
        return mantissa * (2 ** exponent)

    def _linear16_to_float(self, linear16):
        """ Convert a Linear16 formatted value to a float. """
        exponent = self.vout_mode & 0x1F
        if exponent >= 0x10:
            exponent -= 0x20
        return linear16 * (2 ** exponent)

    def _vid_mode_to_float(self, raw_value):
        vid_mode = self.vout_mode & 0x0F
        if vid_mode == 0x04:
            value = 0.5 + (raw_value - 1) * 0.01 # 10mV
        elif vid_mode == 0x07:
            value = 0.25 + (raw_value - 1) * 0.005  # 5mV
        else:
            value = 0.25 + (raw_value - 1) * 0.005  # XmV, fix this !
        return value

    def _direct_to_float(self, raw_value):
        if self.name in ("MPQ2283", "MPQ2285"):
            VOUT_SL = self._read_byte(PMBUS.VOUT_SCALE_LOOP) + 1
            value = ((raw_value * 6.25e-3) + 206.25e-3) * VOUT_SL
        elif self.name == "MPQ72963":
            value = raw_value * 1.5625e-3
        else:
            value = 0 # implement !
        return value

    def _float_to_vid_mode(self, value):
        vid_mode = self.vout_mode & 0x0F
        value *= 1000
        if vid_mode == 0x04:
            raw_value = (((value - 500) / 10) + 1)
        elif vid_mode == 0x07:
            raw_value = (((value - 250) / 5) + 1)
        else:
            raw_value = (((value - 250) / 5) + 1)
        return round(raw_value)

    def _float_to_linear11(value):
        # Initialize the exponent
        exponent = 0
        # Scale the mantissa so that it fits in 11 bits
        mantissa = value
        while mantissa < -1024 or mantissa > 1023:
            mantissa /= 2
            exponent += 1
        
        # If the mantissa is too small, adjust by decreasing the exponent
        while mantissa > -1024 and mantissa < 1024 and exponent > -16:
            mantissa *= 2
            exponent -= 1
        # Round mantissa to the nearest integer
        mantissa = round(mantissa)
        # Handle cases where the mantissa still doesn't fit after rounding
        if mantissa < -1024 or mantissa > 1023:
            raise ValueError("Value cannot be represented in Linear11 format")
        # Encode exponent (5 bits, two's complement)
        encoded_exponent = exponent & 0x1F
        # Encode mantissa (11 bits, two's complement)
        encoded_mantissa = mantissa & 0x07FF
        
        # Combine into a 16-bit value
        linear11_value = (encoded_exponent << 11) | encoded_mantissa
        return linear11_value

    def _float_to_linear16(self, value):
        exponent = self.vout_mode & 0x1F
        if exponent >= 0x10:
            exponent -= 0x20
        return round((value / (2 ** exponent)))

    def _float_to_direct(self, value):
        if self.name in ("MPQ2283", "MPQ2285"):
            VOUT_SL = self._read_byte(PMBUS.VOUT_SCALE_LOOP) + 1
            raw_value = ((value / VOUT_SL) - 206.25e-3) / 6.25e-3
            #value = ((raw_value * 6.25e-3) + 206.25e-3) * VOUT_SL
        elif self.name == "MPQ72963":
            raw_value = value / 5e-3
        else:
            raw_value = 0 # implement !
        return round(raw_value)

    def _value_2_rawvalue(self, value, scaling):
        match scaling:
            case ScalingType.VID:
                return int(self._float_to_vid_mode(value))
            case ScalingType.LINEAR16:
                return int(self._float_to_linear16(value))
            case ScalingType.LINEAR11:
                return int(self._float_to_linear11(value))
            case ScalingType.DIRECT:
                return int(self._float_to_direct(value))
            case _:
                return value / (2 ** self.vout_mode)

    def _rawvalue_2_value(self, raw_value, scaling):
        """Apply scaling to raw PMBus data based on scaling type."""
        match scaling:
            case ScalingType.VID:
                return self._vid_mode_to_float(raw_value)
            case ScalingType.LINEAR16:
                return self._linear16_to_float(raw_value)
            case ScalingType.LINEAR11:
                return self._linear11_to_float(raw_value)
            case ScalingType.DIRECT:
                return self._direct_to_float(raw_value)
            case _:
                return raw_value * (2 ** self.vout_mode)

    def _alert_monitor(self):
        """Monitor ALERT GPIO and handle interrupts."""
        while True:
            self.alert_gpio.poll(timeout=None)
            status_word = self.handle_alert()
            if status_word is not None:
                pm_print(f"Alert! STATUS_WORD: 0x{status_word:04X}")

    def handle_alert(self):
        """Handle PMBus ALERT by checking STATUS_WORD."""
        ara_msg = I2C.Message([0x00], read=True)
        self.i2c.transfer(0x0C, [ara_msg])
        if ara_msg.data[0] == self.addr:
            return self.read_status_word()
        return None

    def read_status_word(self):
        """Read STATUS_WORD for fault info."""
        return self._read_word(PMBUS.STATUS_WORD)

    def _select_page(self):
        """Set the PMBus page for output control."""
        pm_print("set page to 0x{0:02x}".format(self.page))
        self._write_byte(PMBUS.PAGE, self.page)

    def _get_vout_mode(self):
        """Retrieve VOUT_MODE scaling exponent."""
        if self.pmbus_vout_mode <= 0:
            return
        self.vout_mode = self._read_byte(PMBUS.VOUT_MODE)
        pm_print("raw_vout_mode 0x{0:02x}".format(self.vout_mode))

    def _write_word(self, command, value):
        try:
            write_msg = I2C.Message([command, value & 0xFF, (value >> 8) & 0xFF])
            self.i2c.transfer(self.addr, [write_msg])
        except IOError:
            pm_print("IOError: {0}@0x{1:02x}".format(self.i2c.devpath, self.addr))

    def _read_word(self, command):
        word = None
        try:
            write_msg = I2C.Message([command])
            read_msg = I2C.Message([0x00, 0x00], read=True)
            self.i2c.transfer(self.addr, [write_msg, read_msg])
            word = read_msg.data[0] | (read_msg.data[1] << 8)
        except IOError:
            pm_print("IOError: {0}@0x{1:02x}".format(self.i2c.devpath, self.addr))
        return word

    def _write_byte(self, command, value):
        try:
            write_msg = I2C.Message([command, value])
            self.i2c.transfer(self.addr, [write_msg])
        except IOError:
            pm_print("IOError: {0}@0x{1:02x}".format(self.i2c.devpath, self.addr))

    def _read_byte(self, command):
        byte = None
        try:
            write_msg = I2C.Message([command])
            read_msg = I2C.Message([0x00], read=True)
            self.i2c.transfer(self.addr, [write_msg, read_msg])
            byte = read_msg.data[0]
        except IOError:
            pm_print("IOError: {0}@0x{1:02x}".format(self.i2c.devpath, self.addr))
        return byte

class MPSRegulator:
    def __init__(self, device_path, device_address, page=-1, phase=-1, fb_ratio=1, alert_gpio_pin=None):
        """
        Initialize a MPS(MPM54) device with a specific page (output).

        :param device_path: I2C bus number (e.g., 1 for /dev/i2c-1)
        :param device_address: I2C address of the PMIC
        :param page: Page number corresponding to the output
        :param phase: Phase number corresponding to the output
        :param fb_ratio: Feedback Ratio corresponding to the output
        :param alert_gpio_pin: Optional GPIO pin for ALERT (None if not used)
        """
        self.i2c = I2C(device_path)
        self.addr = int(device_address, 0)
        self.page = page
        self.phase = phase
        self.fb_ratio = fb_ratio

        # Initialize ALERT pin if provided
        self.alert_gpio = GPIO(alert_gpio_pin, "in") if alert_gpio_pin is not None else None
        if self.alert_gpio:
            self.alert_gpio.edge = "falling"
            self.alert_thread = threading.Thread(target=self._alert_monitor)
            self.alert_thread.daemon = True
            self.alert_thread.start()

    def __str__(self):
        str_info = {
            k: v
            for k, v in self.__dict__.items()
            if k not in ['i2c'] and v
        }
        return str(str_info)

    def close(self):
        """Clean up resources."""
        self.i2c.close()
        if self.alert_gpio:
            self.alert_gpio.close()

    def enable_output(self):
        pm_print("enable_output")
        """Enable output via OPERATION command."""
        OPERATION = 0x0D
        temp = self._read_byte(OPERATION)
        mask = 0
        if self.page >= 0:
            if self.page == 0:
                mask = 0x40
            else:
                mask = 0x20
        else:
            mask = 0x80
        temp |= mask
        self._write_byte(OPERATION, temp)

    def shutdown_output(self):
        pm_print("shutdown_output")
        OPERATION = 0x0D
        temp = self._read_byte(OPERATION)
        mask = 0
        if self.page >= 0:
            if self.page == 0:
                mask = ~0x40
            else:
                mask = ~0x20
        else:
            mask = ~0x80
        temp &= mask
        self._write_byte(OPERATION, temp)

    def set_voltage(self, value):
        pm_print("set_voltage")
        #self.shutdown_output()
        #VOUT_COMMAND = 0x21
        #raw_value = self._unscaled_value(value, self.vout_scaling)
        #self._write_word(raw_value)
        #self.enable_output()

    def read_all(self):
        values = {}
        values['voltage'] = self.read_voltage()
        values['current'] = self.read_current()
        values['temperature'] = self.read_temperature()
        return values

    def read_voltage(self):
        READ_VOUT = 0x12
        if self.page >= 0:
            READ_VOUT += self.page * 2
        raw_voltage = self._read_byte(READ_VOUT)
        pm_print("raw_voltage(0x{0:02x}) 0x{1:02x}".format(READ_VOUT, raw_voltage))
        voltage = ((raw_voltage * 16) / 1000) / self.fb_ratio # 16mV per LSB and mV to V
        return round(voltage, 4)

    def read_current(self):
        READ_IOUT = 0x13
        if self.page >= 0:
            READ_IOUT += self.page * 2
        raw_current = self._read_byte(READ_IOUT)
        pm_print("raw_current(0x{0:02x}) 0x{1:02x}".format(READ_IOUT, raw_current))
        return ((raw_current * 50) / 1000) # 50mA per LSB and mV to V

    def read_temperature(self):
        """Read and scale output temperature using TEMPERATURE command."""
        """     000: < 80°C
                001: 85°C
                010: 95°C
                011: 105°C
                100: 115°C
                101: 125°C
                110: 135°C
                111: ≥ 140°C """
        TEMPERATURE = 0x11
        raw_temp = self._read_byte(TEMPERATURE)
        return ((((raw_temp & 0xE0) >> 5) * 5) + 80)

    def _write_word(self, command, value):
        try:
            write_msg = I2C.Message([command, value & 0xFF, (value >> 8) & 0xFF])
            self.i2c.transfer(self.addr, [write_msg])
        except IOError:
            pm_print("IOError: {0}@0x{1:02x}".format(self.i2c.devpath, self.addr))

    def _read_word(self, command):
        word = None
        try:
            write_msg = I2C.Message([command])
            read_msg = I2C.Message([0x00, 0x00], read=True)
            self.i2c.transfer(self.addr, [write_msg, read_msg])
            word = read_msg.data[0] | (read_msg.data[1] << 8)
        except IOError:
            pm_print("IOError: {0}@0x{1:02x}".format(self.i2c.devpath, self.addr))
        return word

    def _write_byte(self, command, value):
        try:
            write_msg = I2C.Message([0x00, 0x00])
            self.i2c.transfer(self.addr, [write_msg])
            
            write_msg = I2C.Message([command, value])
            self.i2c.transfer(self.addr, [write_msg])
            
            write_msg = I2C.Message([0x00, 0x01])
            self.i2c.transfer(self.addr, [write_msg])
        except IOError:
            pm_print("IOError: {0}@0x{1:02x}".format(self.i2c.devpath, self.addr))

    def _read_byte(self, command):
        byte = None
        try:
            write_msg = I2C.Message([command])
            read_msg = I2C.Message([0x00], read=True)
            self.i2c.transfer(self.addr, [write_msg, read_msg])
            byte = read_msg.data[0]
        except:
            pm_print("IOError: {0}@0x{1:02x}".format(self.i2c.devpath, self.addr))
        return byte


# Copyright (C) 2021-2022 Xilinx, Inc.  All rights reserved.
# Copyright (C) 2022-2024 Advanced Micro Devices, Inc. All Rights Reserved.
# SPDX-License-Identifier: BSD-3-Clause

__author__ = "Anish Kadamathikuttiyil Karthikeyan Pillai"
__copyright__ = "Copyright 2021, Xilinx"

import sys
def usage():
    # sys.argv[0] - usage python file name
    # sys.argv[1] - xclient/xcffi/xpyro
    # sys.argv[2] - host/board
    # sys.argv[3] - ip_address
    # sys.argv[4] - port
    print(f"Usage: The application work in three modes xclient, xcffi and xpyro\n"
          f"python3 {sys.argv[0]} xclient host/board ip_address port => Runs at host or board with xclient\n"
          f"sudo python3 {sys.argv[0]} xcffi => Runs at board with xcffi\n"
          f"python3 {sys.argv[0]} xpyro host/board ip_address port => Runs at host or board with xpyro\n"
          f"Example: python3 {sys.argv[0]} xclient host 169.254.10.2 9090")

# 'xcffi' option will run only in the board
if (len(sys.argv) ==  2):
    if (sys.argv[1] != 'xcffi'):
        usage()
        sys.exit()

elif (len(sys.argv) !=  5):
    usage()
    sys.exit()

# The 'xclient' option will run both in host and board
if (sys.argv[1] == 'xclient'):
    # For 'xclient' option the path of the xclient is required
    # The relative path in the host is given for host case
    if (sys.argv[2] == 'host'):
        sys.path.append('../../../xclient/dfe')
    # The location of xclient in the filesystem is given for the board case
    elif (sys.argv[2] == 'board'):
        sys.path.append('/usr/share/raft/xclient/dfe')
    else:
        usage()
        sys.exit()
    import prach_client
    handle = prach_client.prach
    ip_address = sys.argv[3]
    port = sys.argv[4]

elif (sys.argv[1] == 'xcffi'):
    # For 'xcffi' option the path of the cffi python code is required
    sys.path.append('/usr/share/raft/xserver/xcffi/drv_api/dfe')
    from prach_server import PRACH
    handle = PRACH()

# The 'xpyro' option will run both in host and board
elif (sys.argv[1] == 'xpyro'):
    ip_address = sys.argv[3]
    port = sys.argv[4]
    import Pyro4
    # Prepare the uri needed for pyro communication
    uri = f"PYRO:PRACH@{ip_address}:{port}"
    handle = Pyro4.Proxy(uri)
else:
    usage()
    sys.exit()

if (sys.argv[1] == 'xclient'):
    #SetIpAndPort
    handle.SetIpAndPort(ip_address, port)
    #GetPythonLogLevels
    PythonLogLevels = handle.GetPythonLogLevels()
    #SetClientLogLevel
    handle.SetClientLogLevel(PythonLogLevels["DEBUG"])

#Description:
#   Return the logging levels supported by logging library in python
#Input Arguments:
#  None
#Return:
#   Dictionary showing the log levels supported by logging library
#GetPythonLogLevels
PythonLogLevels = handle.GetPythonLogLevels()

#Description:
#   Set the python log level to the given level
#Input Arguments:
#    Log level to set
#Return:
#   None
#SetServerLogLevel
handle.SetServerLogLevel(PythonLogLevels["DEBUG"])

#Description:
#   Set the metal log level to the given level
#Input Arguments:
#   Log level to set
#Return:
#   None
#SetMetalLogLevel
metal_log_level = handle.GetEnum_metal_log_level()
handle.SetMetalLogLevel(metal_log_level["METAL_LOG_EMERGENCY"])

#Description:
#   Return Dictionary with all PRACH macros in the prach header file
#Input Arguments:
#   None
#Return:
#   Dictionary with all PRACH macros in the prach header file
# GetPrachMacro
prach_macro = handle.GetPrachMacro()
print(prach_macro["XDFEPRACH_MAX_NUM_INSTANCES"])
print(prach_macro["XST_SUCCESS"])
print(prach_macro["XST_FAILURE"])
print(prach_macro["XDFEPRACH_NODE_NAME_MAX_LENGTH"])
print(prach_macro["XDFEPRACH_CC_NUM_MAX"])
print(prach_macro["XDFEPRACH_RC_NUM_MAX"])
print(prach_macro["XDFEPRACH_NCO_NUM_MAX"])
print(prach_macro["XDFEPRACH_ANT_NUM_MAX"])
print(prach_macro["XDFEPRACH_SEQ_LENGTH_MAX"])
print(prach_macro["XDFEPRACH_BAND_ID_MAX"])

#Description:
#   Return Dictionary equivalent of enum XDfePrach_StateId
#Input Arguments:
#   None
#Return:
#   Dictionary equivalent of enum XDfePrach_StateId
# GetEnum_XDfePrach_StateId
XDfePrach_StateId = handle.GetEnum_XDfePrach_StateId()

#Description:
#   Return Dictionary equivalent of structure XDfePrach_InternalChannelCfg
#Input Arguments:
#   None
#Return:
#   Dictionary equivalent of structure XDfePrach_InternalChannelCfg
# GetStruct_XDfePrach_InternalChannelCfg
XDfePrach_InternalChannelCfg = handle.GetStruct_XDfePrach_InternalChannelCfg()

#Description:
#   Return Dictionary equivalent of structure XDfePrach_ChannelCfg
#Input Arguments:
#   None
#Return:
#   Dictionary equivalent of structure XDfePrach_ChannelCfg
# GetStruct_XDfePrach_ChannelCfg
XDfePrach_ChannelCfg = handle.GetStruct_XDfePrach_ChannelCfg()

#Description:
#   Return Dictionary equivalent of enum metal_log_level
#Input Arguments:
#   None
#Return:
#   Dictionary equivalent of enum metal_log_level
# GetEnum_metal_log_level
metal_log_level = handle.GetEnum_metal_log_level()

#Description:
#   API initialise an instance of the driver.
#   Traverse "/sys/bus/platform/device" directory (in Linux), to find registered
#   PRACH device with the name DeviceNodeName. The first available slot in
#   the instance array XDfePrach_Prach[] will be taken as a DeviceNodeName
#   object. On success it moves the state machine to a Ready state, while on
#   failure stays in a Not Ready state.
#C header declaration:
#   XDfePrach *XDfePrach_InstanceInit(const char *DeviceNodeName);
#Input Arguments:
#   DeviceNodeName: device node name.
#Return:
#   ret - 0 on success, 1 on failure
#   device_id - integer handle to initalized instance
#   DeviceNodeNameRet - device node name returned from driver
#                       which will be same as the passed value

DeviceNodeName = "a7e00000.xdfe_nr_prach"
ret, device_id, DeviceNodeName = handle.XDfePrach_InstanceInit(DeviceNodeName)

#Description:
#   Writes a value to register in a PRACH instance.
#C header declaration:
#   void XDfePrach_WriteReg(const XDfePrach *InstancePtr, u32 AddrOffset, u32 Data);
#Input Arguments:
#   device_id: id of the opened device.
#   AddrOffset: is address offset relative to instance base address.
#   Data: is value to be written.
#Return:
#   None

handle.XDfePrach_WriteReg(device_id, 0x40, 0)

#Description:
#   Reads a value the register in a PRACH instance.
#C header declaration:
#   u32 XDfePrach_ReadReg(const XDfePrach *InstancePtr, u32 AddrOffset);
#Input Arguments:
#   device_id: id of the opened device.
#   AddrOffset: is address offset relative to instance base address.
#Return:
#   ret: Register value.

ret = handle.XDfePrach_ReadReg(device_id, 0)

#Description:
#   Resets PRACH and puts block into a reset state.
#C header declaration:
#   void XDfePrach_Reset(XDfePrach *InstancePtr);
#Input Arguments:
#   device_id: id of the opened device.
#Return:
#   None

handle.XDfePrach_Reset(device_id)

#Description:
#   Reads configuration from device tree/xparameters.h and IP registers.
#   Removes S/W reset and moves the state machine to a Configured state.
#C header declaration:
#   void XDfePrach_Configure(XDfePrach *InstancePtr, XDfePrach_Cfg *Cfg);
#Input Arguments:
#   device_id: id of the opened device.
#   Cfg: configuration data container.
#Return:
#   Cfg: configuration data container.

Cfg_in = handle.GetStruct_XDfePrach_Cfg()
Cfg_out = handle.XDfePrach_Configure(device_id, Cfg_in)

#Description:
#   DFE PRACH driver one time initialisation also moves the state machine to
#   an Initialised state.
#C header declaration:
#   void XDfePrach_Initialize(XDfePrach *InstancePtr, const XDfePrach_Init *Init);
#Input Arguments:
#   device_id: id of the opened device.
#   Init: initialisation data container.
#Return:
#   None

Init_in = handle.GetStruct_XDfePrach_Init()
Init_in['Sequence'][0]['Length']= 6
Init_out = handle.XDfePrach_Initialize(device_id, Init_in)

#Description:
#   Set trigger configuration.
#C header declaration:
#   void XDfePrach_SetTriggersCfg(const XDfePrach *InstancePtr,
# 	XDfePrach_TriggerCfg *TriggerCfg)
#Input Arguments:
#   device_id: id of the opened device.
#   TriggerCfg: Trigger configuration container.
#Return:
#   TriggerCfg: Trigger configuration container.

TriggerCfg_in = handle.GetStruct_XDfePrach_TriggerCfg()
TriggerCfg_in["FrameInit"][0]["Mode"] = 1
TriggerCfg_in["FrameInit"][1]["Mode"] = 1
TriggerCfg_in["FrameInit"][2]["Mode"] = 1
TriggerCfg_out = handle.XDfePrach_SetTriggersCfg(device_id, TriggerCfg_in)

#Description:
#   Activates PRACH and moves the state machine to an Activated state.
#C header declaration:
#   void XDfePrach_Activate(XDfePrach *InstancePtr, bool EnableLowPower);
#Input Arguments:
#   device_id: id of the opened device.
#   EnableLowPower: flag indicating low power.
#Return:
#   None

handle.XDfePrach_Activate(device_id, 0)

#Description:
#   Gets a state machine state id.
#C header declaration:
#   XDfePrach_StateId XDfePrach_GetStateID(XDfePrach *InstancePtr);
#Input Arguments:
#   device_id: id of the opened device.
#Return:
#   StateId: State machine StateID as a string

StateId = handle.XDfePrach_GetStateID(device_id)
print(StateId)

#Description:
#   Returns the current CC configuration
#C header declaration:
#   void XDfePrach_GetCurrentCCCfg(const XDfePrach *InstancePtr, XDfePrach_CCCfg *CCCfg);
#Input Arguments:
#   device_id: id of the opened device.
#   CurrCCCfg: CC configuration container.
#Return:
#   CurrCCCfg: CC configuration container

CurrCCCfg = handle.GetStruct_XDfePrach_CCCfg()
CurrentCCCfg = handle.XDfePrach_GetCurrentCCCfg(device_id, CurrCCCfg)

#Description:
#   Returns configuration structure CCCfg with CCCfg->Sequence.Length value set
#   in XDfePrach_Configure(), array CCCfg->Sequence.CCID[] members are set to not
#   used value (-1) and the other CCCfg members are set to 0.
#C header declaration:
#   void XDfePrach_GetEmptyCCCfg(const XDfePrach *InstancePtr, XDfePrach_CCCfg *CCCfg);
#Input Arguments:
#   device_id: id of the opened device.
#Return:
#   CCCfg: CC configuration container

CCCfg = handle.XDfePrach_GetEmptyCCCfg(device_id)

"""
Returns the current CCID carrier configuration.
:param device_id: id of the opened device.
:param CCCfg: component carrier (CC) configuration container.
:param CCID: Channel ID.
:param BandId: Band Id.
:return: CCSeqBitmap: CC slot position container.
CarrierCfg: CC configuration container.
"""
CCCfg = handle.GetStruct_XDfePrach_CCCfg()
CCID = 0
BandId = 0
CCSeqBitmap, CarrierCfg = handle.XDfePrach_GetCarrierCfgMB(device_id, CCCfg, CCID, BandId)

#Description:
#   Returns the current CCID carrier configuration.
#C header declaration:
#   void XDfePrach_GetCarrierCfg(const XDfePrach *InstancePtr, XDfePrach_CCCfg *CCCfg,
#   s32 CCID, u32 *CCSeqBitmap, XDfePrach_CarrierCfg *CarrierCfg);
#Input Arguments:
#   device_id: id of the opened device.
#   CCCfg: component carrier (CC) configuration container.
#   CCID: Channel ID.
#Return:
#   CCSeqBitmap: CC slot position container.
#   CarrierCfg: CC configuration container.


CCCfg = handle.GetStruct_XDfePrach_CCCfg()
CCSeqBitmap, CarrierCfg = handle.XDfePrach_GetCarrierCfg(device_id, CCCfg, 0)

#Description:
#   Set antenna configuration in CC configuration container.
#C header declaration:
#   void XDfePrach_SetAntennaCfgInCCCfg(const XDfePrach *InstancePtr,
#   XDfePrach_CCCfg *CCCfg, u32 *AntennaCfg);
#Input Arguments:
#   device_id: id of the opened device.
#   CCCfg: component carrier (CC) configuration container.
#   AntennaCfg: Array of all antenna configurations.
#Return:
#   CCCfg: component carrier (CC) configuration container.


AntennaCfg=[1,1,1,1,1,1,0,0]
CCCfg = handle.XDfePrach_SetAntennaCfgInCCCfg(device_id, CCCfg, AntennaCfg)

"""
Adds specified CCID, with specified configuration, to a local CC
configuration structure for the chosen Band.
If there is insufficient capacity for the new CC the function will return
an error.
Initiates CC update (enable CCUpdate trigger TUSER Single Shot).
:param device_id: id of the opened device.
:param CCCfg: component carrier (CC) configuration container.
:param CCID: Channel ID.
:param CCSeqBitmap: CC slot position container.
:param CarrierCfg: CC configuration container.
:param BandId: Band Id.
:return: ret: XST_SUCCESS if successful, XST_FAILURE if error occurs.
CCCfg: component carrier (CC) configuration container.
"""
CCCfg = handle.GetStruct_XDfePrach_CCCfg()
CarrierCfg = handle.GetStruct_XDfePrach_CarrierCfg()
CCID = 0x5
CCSeqBitmap = 0x1
CarrierCfg['SCS'] = 0
BandId = 0
ret, CCCfg = handle.XDfePrach_AddCCtoCCCfgMB(device_id, CCCfg, CCID, CCSeqBitmap, CarrierCfg, BandId)

#Description:
#   Adds specified CCID, with specified configuration, to a local CC
#   configuration structure.
#   If there is insufficient capacity for the new CC the function will return
#   an error.
#   Initiates CC update (enable CCUpdate trigger TUSER Single Shot).
#C header declaration:
#   u32 XDfePrach_AddCCtoCCCfg(XDfePrach *InstancePtr, XDfePrach_CCCfg *CCCfg, s32 CCID,
#   u32 CCSeqBitmap, const XDfePrach_CarrierCfg *CarrierCfg);
#Input Arguments:
#   device_id: id of the opened device.
#   CCCfg: component carrier (CC) configuration container.
#   CCID: Channel ID.
#   CCSeqBitmap: CC slot position container.
#   CarrierCfg: CC configuration container.
#Return:
#   ret: XST_SUCCESS if successful, XST_FAILURE if error occurs.
#   CCCfg: component carrier (CC) configuration container.

CCCfg = handle.GetStruct_XDfePrach_CCCfg()
CarrierCfg = handle.GetStruct_XDfePrach_CarrierCfg()
CCID = 0x5
CCSeqBitmap = 0x1
CarrierCfg['SCS'] = 0
ret, CCCfg = handle.XDfePrach_AddCCtoCCCfg(device_id, CCCfg, CCID, CCSeqBitmap, CarrierCfg)

"""
Removes specified CCID from a local CC configuration structure for
selected band.
:param device_id: id of the opened device.
:param CCCfg: component carrier (CC) configuration container.
:param CCID: Channel ID.
:param BandId: Band Id.
:return: CCCfg: component carrier (CC) configuration container.
"""
CCCfg = handle.GetStruct_XDfePrach_CCCfg()
CCID = 0
BandId = 0
CCCfg = handle.XDfePrach_RemoveCCfromCCCfgMB(device_id, CCCfg, CCID, BandId)

#Description:
#   Removes specified CCID from a local CC configuration structure.
#C header declaration:
#   void XDfePrach_RemoveCCfromCCCfg(XDfePrach *InstancePtr, XDfePrach_CCCfg *CCCfg,
# 	s32 CCID);
#Input Arguments:
#   device_id: id of the opened device.
#   CCCfg: component carrier (CC) configuration container.
#   CCID: Channel ID.
#Return:
#   CCCfg: component carrier (CC) configuration container.

CCCfg = handle.GetStruct_XDfePrach_CCCfg()
CCCfg = handle.XDfePrach_RemoveCCfromCCCfg(device_id, CCCfg, 0)

"""
Updates specified CCID, with specified configuration to a local CC
configuration structure for selected band.
If there is insufficient capacity for the new CC the function will return
an error.
:param device_id: id of the opened device.
:param CCCfg: component carrier (CC) configuration container.
:param CCID: Channel ID.
:param CarrierCfg: CC configuration container.
:param BandId: Band Id.
:return: CCCfg: component carrier (CC) configuration container.
"""
CCCfg = handle.GetStruct_XDfePrach_CCCfg()
CarrierCfg = handle.GetStruct_XDfePrach_CarrierCfg()
CCID = 0
BandId = 0
CCCfg = handle.XDfePrach_UpdateCCinCCCfgMB(device_id, CCCfg, CCID, CarrierCfg, BandId)

#Description:
#   Updates specified CCID, with specified configuration to a local CC
#   configuration structure.
#   If there is insufficient capacity for the new CC the function will return
#   an error.
#C header declaration:
#   void XDfePrach_UpdateCCinCCCfg(const XDfePrach *InstancePtr, XDfePrach_CCCfg *CCCfg,
#   s32 CCID, const XDfePrach_CarrierCfg *CarrierCfg);
#Input Arguments:
#   device_id: id of the opened device.
#   CCCfg: component carrier (CC) configuration container.
#   CCID: Channel ID.
#   CarrierCfg: CC configuration container.
#Return:
#   CCCfg: component carrier (CC) configuration container.

CCCfg = handle.GetStruct_XDfePrach_CCCfg()
CarrierCfg = handle.GetStruct_XDfePrach_CarrierCfg()
CCCfg = handle.XDfePrach_UpdateCCinCCCfg(device_id, CCCfg, 0, CarrierCfg)

#Description:
#   Add specified CCID, with specified configuration.
#C header declaration:
#   u32 XDfePrach_AddCC(const XDfePrach *InstancePtr, u32 CCId,
#   const XDfePrach_CarrierCfg *CarrierCfg);
#Input Arguments:
#   device_id: id of the opened device.
#   CCID: is a Channel ID.
#   BitSequence: up to 16 defined slots into which a CC can be
#                allocated. The number of slots can be from 1 to 16 depending on
#                system initialization. The number of slots is defined by the
#                "sequence length" parameter which is provided during initialization.
#                The Bit offset within the CCSeqBitmap indicates the equivalent
#                Slot number to allocate. e.g. 0x0003  means the caller wants the
#                passed component carrier (CC) to be allocated to slots 0 and 1.
#   CarrierCfg: is a CC configuration container.
#Return:
#   ret: XST_SUCCESS if successful, XST_FAILURE if error occurs.
#   CarrierCfg: is a CC configuration container.

CarrierCfg_in = handle.GetStruct_XDfePrach_CarrierCfg()
BitSequence = 0xb
ret, CarrierCfg_out = handle.XDfePrach_AddCC(device_id, 0, BitSequence, CarrierCfg_in)

#Description:
#   Updates a CCID sequence.
#C header declaration:
#   u32 XDfePrach_UpdateCC(const XDfePrach *InstancePtr, u32 CCId,
#   const XDfePrach_CarrierCfg *CarrierCfg);
#Input Arguments:
#   device_id: id of the opened device.
#   CCID: is a Channel ID.
#   CarrierCfg: is carrier data container.
#Return:
#   ret: XST_SUCCESS if successful, XST_FAILURE if error occurs.

CarrierCfg_in = handle.GetStruct_XDfePrach_CarrierCfg()
ret = handle.XDfePrach_UpdateCC(device_id, 0, CarrierCfg_in)

#Description:
#   Reads all of the RC configuration back.
#C header declaration:
#   void XDfePrach_GetCurrentRCCfg(const XDfePrach *InstancePtr,
#   XDfePrach_RCCfg *RCCfg);
#Input Arguments:
#   device_id: id of the opened device.
#Return:
#   RCCfg: RC configuration container.

RCCfg = handle.XDfePrach_GetCurrentRCCfg(device_id)

#Description:
#   Returns the empty CC configuration.
#C header declaration:
#   void XDfePrach_GetEmptyRCCfg(const XDfePrach *InstancePtr,
#   XDfePrach_RCCfg *RCCfg);
#Input Arguments:
#   device_id: id of the opened device.
#Return:
#   RCCfg: RC configuration container.

RCCfg = handle.XDfePrach_GetEmptyRCCfg(device_id)

#Description:
#   Gets RACH channel configuration.
#C header declaration:
#   void XDfePrach_GetChannelCfg(const XDfePrach *InstancePtr, XDfePrach_RCCfg *RCCfg,
#   s32 RCId, XDfePrach_ChannelCfg *ChannelCfg)
#Input Arguments:
#   device_id: id of the opened device.
#   RCCfg: RC configuration container.
#   RCId: Chosen RACH channel Id.
#Return:
#   ret: XST_SUCCESS on success, XST_FAILURE on failure
#   ChannelCfg: RACH channel container.

CurrentRCCfg = handle.GetStruct_XDfePrach_RCCfg()
ChannelCfg = handle.XDfePrach_GetChannelCfg(device_id, CurrentRCCfg, 0)

#Description:
#   Adds a new RC entry to the RC_CONFIGURATION. RCId must be same as the
#   physical channel RachChan.
#C header declaration:
#   u32 XDfePrach_AddRCtoRCCfg(const XDfePrach *InstancePtr, XDfePrach_RCCfg *CurrentRCCfg,
#   s32 CCID, u32 RCId, u32 RachChan, XDfePrach_DDCCfg *DdcCfg, XDfePrach_NCO *NcoCfg,
#	XDfePrach_Schedule *StaticSchedule, u32 BandId);
#Input Arguments:
#   device_id: id of the opened device.
#   CurrentRCCfg: current RACH configuration container
#   CCID: is CC Id.
#   RCId: is RC Id.
#   RachChan: is RACH channel.
#   DdcCfg: is DDC data container.
#   NcoCfg: is NCO data container.
#   Schedule: is Schedule data container.
#   BandId: is band id.
#Return:
#   ret: XST_SUCCESS on success, XST_FAILURE on failure
#   CurrentRCCfg: current RACH configuration container

# RC depends on CC settings, that is why we run AddCC first
CCCfg = handle.GetStruct_XDfePrach_CCCfg()
CarrierCfg = handle.GetStruct_XDfePrach_CarrierCfg()
CCID = 0x5
CCSeqBitmap = 0x1
CarrierCfg['SCS'] = 0
BandId = 0
ret, NextCCCfg = handle.XDfePrach_AddCCtoCCCfgMB(device_id, CCCfg, CCID, CCSeqBitmap, CarrierCfg, BandId)

#Description:
#   Adds a new RC entry to the RC_CONFIGURATION. RCId must be same as the
#   physical channel RachChan.
#C header declaration:
#   u32 XDfePrach_AddRCtoRCCfg(const XDfePrach *InstancePtr, XDfePrach_RCCfg *CurrentRCCfg,
#   s32 CCID, u32 RCId, u32 RachChan, XDfePrach_DDCCfg *DdcCfg, XDfePrach_NCO *NcoCfg,
#	XDfePrach_Schedule *StaticSchedule);
#Input Arguments:
#   device_id: id of the opened device.
#   CurrentRCCfg: current RACH configuration container
#   CCID: is CC Id.
#   RCId: is RC Id.
#   RachChan: is RACH channel.
#   DdcCfg: is DDC data container.
#   NcoCfg: is NCO data container.
#   Schedule: is Schedule data container.
#Return:
#   ret: XST_SUCCESS on success, XST_FAILURE on failure
#   CurrentRCCfg: current RACH configuration container

# RC depends on CC settings, that is why we run AddCC first
CCCfg = handle.GetStruct_XDfePrach_CCCfg()
CarrierCfg = handle.GetStruct_XDfePrach_CarrierCfg()
CCID = 0x5
CCSeqBitmap = 0x1
CarrierCfg['SCS'] = 0
ret, NextCCCfg = handle.XDfePrach_AddCCtoCCCfg(device_id, CCCfg, CCID, CCSeqBitmap, CarrierCfg)

CurrentRCCfg = handle.GetStruct_XDfePrach_RCCfg()
DdcCfg = handle.GetStruct_XDfePrach_DDCCfg()
NcoCfg = handle.GetStruct_XDfePrach_NCO()
StaticSchedule = handle.GetStruct_XDfePrach_Schedule()
ret, CurrentCCCfg = handle.XDfePrach_AddRCtoRCCfg(device_id, CurrentRCCfg, 0, 0,
                                           0, DdcCfg, NcoCfg, StaticSchedule, NextCCCfg)
#ret, CurrentCCCfg = handle.XDfePrach_AddRCtoRCCfgDynamic(device_id, CurrentRCCfg, 0, 0, 0, NextCCCfg, 0)
#ret, CurrentCCCfg = handle.XDfePrach_AddRCtoRCCfgMBDynamic(device_id, CurrentRCCfg, 0, 0, 0, NextCCCfg, 0)

#Description:
#   Removes an RC configuration entry from the RC_CONFIGURATION. RCId must be
#   same as the physical channel RachChan.
#C header declaration:
#   u32 XDfePrach_RemoveRCfromRCCfg(const XDfePrach *InstancePtr,
#   XDfePrach_RCCfg *CurrentRCCfg, u32 RCId);
#Input Arguments:
#   device_id: id of the opened device.
#   CurrentRCCfg: current PRACH configuration container
#   RCId: is RC Id.
#Return:
#   ret: XST_SUCCESS on success, XST_FAILURE on failure
#   CurrentRCCfg: current PRACH configuration container

CurrentRCCfg = handle.GetStruct_XDfePrach_RCCfg()
ret, CurrentRCCfg = handle.XDfePrach_RemoveRCfromRCCfg(device_id, CurrentRCCfg, 0)

#Description:
#   Updates an RC entry to the RC_CONFIGURATION.
#C header declaration:
#   void XDfePrach_UpdateRCinRCCfg(const XDfePrach *InstancePtr, XDfePrach_RCCfg *CurrentRCCfg,
#   s32 CCID, u32 RCId, u32 RachChan, XDfePrach_DDCCfg *DdcCfg, XDfePrach_NCO *NcoCfg,
#	XDfePrach_Schedule *StaticSchedule, u32 BandId);
#Input Arguments:
#   device_id: id of the opened device.
#   CurrentRCCfg: current PRACH configuration container
#   CCID: is CC Id.
#   RCId: is RC Id.
#   RachChan: is PRACH channel.
#   DdcCfg: is DDC data container.
#   NcoCfg: is NCO data container.
#   Schedule: is Schedule data container.
#   BandId: is band id.
#Return:
#   CurrentRCCfg: current PRACH configuration container

CurrentRCCfg = handle.GetStruct_XDfePrach_RCCfg()
DdcCfg = handle.GetStruct_XDfePrach_DDCCfg()
NcoCfg = handle.GetStruct_XDfePrach_NCO()
StaticSchedule = handle.GetStruct_XDfePrach_Schedule()
CurrentRCCfg = handle.XDfePrach_UpdateRCinRCCfgMB(device_id, CurrentRCCfg, 0, 0, 0,
                                         DdcCfg, NcoCfg, StaticSchedule, NextCCCfg, BandId)

#Description:
#   Updates an RC entry to the RC_CONFIGURATION.
#C header declaration:
#   void XDfePrach_UpdateRCinRCCfg(const XDfePrach *InstancePtr, XDfePrach_RCCfg *CurrentRCCfg,
#   s32 CCID, u32 RCId, u32 RachChan, XDfePrach_DDCCfg *DdcCfg, XDfePrach_NCO *NcoCfg,
#	XDfePrach_Schedule *StaticSchedule);
#Input Arguments:
#   device_id: id of the opened device.
#   CurrentRCCfg: current PRACH configuration container
#   CCID: is CC Id.
#   RCId: is RC Id.
#   RachChan: is PRACH channel.
#   DdcCfg: is DDC data container.
#   NcoCfg: is NCO data container.
#   Schedule: is Schedule data container.
#Return:
#   CurrentRCCfg: current PRACH configuration container

CurrentRCCfg = handle.GetStruct_XDfePrach_RCCfg()
DdcCfg = handle.GetStruct_XDfePrach_DDCCfg()
NcoCfg = handle.GetStruct_XDfePrach_NCO()
StaticSchedule = handle.GetStruct_XDfePrach_Schedule()
CurrentRCCfg = handle.XDfePrach_UpdateRCinRCCfg(device_id, CurrentRCCfg, 0, 0, 0,
                                         DdcCfg, NcoCfg, StaticSchedule, NextCCCfg)

#Description:
#   Writes local CC configuration to the shadow (NEXT) registers and triggers
#   copying from shadow to operational registers.
#C header declaration:
#   u32 XDfePrach_SetNextCfg(const XDfePrach *InstancePtr, XDfePrach_RCCfg *NextRCCfg);
#Input Arguments:
#   device_id: id of the opened device.
#   NextCCCfg: a CC configuration container.
#   NextRCCfg: a RC configuration container.
#Return:
#   ret: XST_SUCCESS on success, XST_FAILURE on failure

NextCCCfg = handle.GetStruct_XDfePrach_CCCfg()
NextRCCfg = handle.GetStruct_XDfePrach_RCCfg()
ret = handle.XDfePrach_SetNextCfg(device_id, NextCCCfg, NextRCCfg)

#Description:
#   Updates an RC entry to the RC_CONFIGURATION. RCId must be same as the
#   physical channel RachChan.
#C header declaration:
#   u32 XDfePrach_UpdateRCCfg(const XDfePrach *InstancePtr, s32 CCID, u32 RCId, u32 RachChan,
#   XDfePrach_DDCCfg *DdcCfg, XDfePrach_NCO *NcoCfg, XDfePrach_Schedule *StaticSchedule);
#Input Arguments:
#   device_id: id of the opened device.
#   CCID: is CC Id.
#   RCId: is RC Id.
#   RachChan: is PRACH channel.
#   DdcCfg: is DDC data container.
#   NcoCfg: is NCO data container.
#   Schedule: is Schedule data container.
#Return:
#   ret: XST_SUCCESS on success, XST_FAILURE on failure

DdcCfg = handle.GetStruct_XDfePrach_DDCCfg()
NcoCfg = handle.GetStruct_XDfePrach_NCO()
StaticSchedule = handle.GetStruct_XDfePrach_Schedule()
ret = handle.XDfePrach_UpdateRCCfg(device_id, 0, 0, 0, DdcCfg, NcoCfg, StaticSchedule)

#Description:
#   Adds a new RC entry to the RC_CONFIGURATION. RCId must be same as the
#   physical channel RachChan.
#C header declaration:
#   u32 XDfePrach_AddRCCfg(const XDfePrach *InstancePtr, #u32 CCId, u32 RCId, u32 RachChan,
#   XDfePrach_DDCCfg *DdcCfg,XDfePrach_NCO *NcoCfg, XDfePrach_Schedule *StaticSchedule);
#Input Arguments:
#   device_id: id of the opened device.
#   CCID: is CC Id.
#   RCId: is RC Id.
#   RachChan: is RACH channel.
#   DdcCfg: is DDC data container.
#   NcoCfg: is NCO data container.
#   StaticSchedule: is Schedule data container.
#Return:
#   ret: XST_SUCCESS on succes, XST_FAILURE on failure
#   DdcCfg: is DDC data container.
#   NcoCfg: is NCO data container.
#   Schedule: is Schedule data container.

# AddCC first
CarrierCfg_in = handle.GetStruct_XDfePrach_CarrierCfg()
CCID = 0x0
CCSeqBitmap = 0x1
CarrierCfg['SCS'] = 0
ret, CarrierCfg_out = handle.XDfePrach_AddCC(device_id, CCID, BitSequence, CarrierCfg_in)
DdcCfg = {"DecimationRate": 0, "UserSCS": 0, "RachGain": [0, 0, 0, 0, 0, 0]}
NcoCfg = {
    "PhaseOffset": 0,
    "PhaseAcc": 0,
    "DualModCount": 0,
    "DualModSel": 0,
    "Frequency": 0,
    "NcoGain": 0,
}
StaticSchedule = {
    "PatternPeriod": 0,
    "FrameID": 0,
    "SubframeID": 0,
    "SlotId": 0,
    "Duration": 0,
    "Repeats": 0,
}
ret, DdcCfg, NcoCfg, StaticSchedule = handle.XDfePrach_AddRCCfg(
    device_id, 0, 0, 0, DdcCfg, NcoCfg, StaticSchedule
)

#Description:
#   Move specified RCID from one NCO & Decimation Channel to another NCO &&
#   Decimation Channel.
#C header declaration:
#   u32 XDfePrach_MoveRC(const XDfePrach *InstancePtr, u32 RCId, u32 ToChannel);
#Input Arguments:
#   device_id: id of the opened device.
#   RCId: is RC Id.
#   ToChannel: is destination channel Id.
#Return:
#   ret: XST_SUCCESS on succes, XST_FAILURE on failure

ret = handle.XDfePrach_MoveRC(device_id, 0, 0)

#Description:
#   Return current trigger configuration.
#C header declaration:
#   void XDfePrach_GetTriggersCfg(const XDfePrach *InstancePtr,
#   XDfePrach_TriggerCfg *TriggerCfg);
#Input Arguments:
#   device_id: id of the opened device.
#Return:
#   TriggerCfg: Trigger configuration container

TriggerCfg_out = handle.XDfePrach_GetTriggersCfg(device_id)

#Description:
#   Get specified CCID carrier configuration from either Current or Next.
#C header declaration:
#   void XDfePrach_GetCC(const XDfePrach *InstancePtr, u32 CCID,
# 	XDfePrach_CarrierCfg *CarrierCfg);
#Input Arguments:
#   device_id: id of the opened device.
#   Next: is next or current data flag.
#   CCID: is component carrier id number.
#Return:
#   CarrierCfg: Carrier config container.

CarrierCfg_out = handle.XDfePrach_GetCC(device_id, 0, 0)

#Description:
#   Get specified CCID carrier configuration from either Current or Next
#   for selected band in multi-band mode.
#C header declaration:
#   void XDfePrach_GetCCMB(const XDfePrach *InstancePtr, u32 CCID,
# 	XDfePrach_CarrierCfg *CarrierCfg, u32 BandId);
#Input Arguments:
#   device_id: id of the opened device.
#   Next: is next or current data flag.
#   CCID: is component carrier id number.
#   BandId: is band id.
#Return:
#   CarrierCfg: Carrier config container.

CarrierCfg_out = handle.XDfePrach_GetCCMB(device_id, 0, 0, 0)

#Description:
#   Get PRACH Status.
#C header declaration:
#   void XDfePrach_GetStatus(const XDfePrach *InstancePtr, XDfePrach_Status *Status);
#Input Arguments:
#   device_id: id of the opened device.
#Return:
#   Status: Status data container.

Status = handle.XDfePrach_GetStatus(device_id)

#Description:
#   Clear the PRACH status registers.
#C header declaration:
#   void XDfePrach_ClearStatus(const XDfePrach *InstancePtr);
#Input Arguments:
#   device_id: id of the opened device.
#Return:
#   None


handle.XDfePrach_ClearStatus(device_id)

#Description:
#   Captures phase for all phase accumulators in associated AXI-lite registers.
#C header declaration:
#   void XDfePrach_CapturePhase(const XDfePrach *InstancePtr);
#Input Arguments:
#   device_id: id of the opened device.
#Return:
#   None

handle.XDfePrach_CapturePhase(device_id)

#Description:
#   Reads the captured phase for a given Rach Channel.
#C header declaration:
#   void XDfePrach_GetCapturePhase(const XDfePrach *InstancePtr, u32 RachChan,
# 	XDfePrach_NCO *CapturedPhase);
#Input Arguments:
#   device_id: id of the opened device.
#   RachChan: is RACH channel Id.
#Return:
#   NCO data container.

CapturedPhase = handle.XDfePrach_GetCapturePhase(device_id, 0)

#Description:
#   Gets interrupt mask status.
#C header declaration:
#   void XDfePrach_GetInterruptMask(const XDfePrach *InstancePtr,
# 	XDfePrach_InterruptMask *Mask);
#Input Arguments:
#   device_id: id of the opened device.
#Return:
#   Mask: interrupt masks container.

InterruptMask_out = handle.XDfePrach_GetInterruptMask(device_id)

#Description:
#   Sets interrupt mask.
#C header declaration:
#   void XDfePrach_SetInterruptMask(const XDfePrach *InstancePtr,
# 	const XDfePrach_InterruptMask *Mask);
#Input Arguments:
#   device_id: id of the opened device.
#   Mask: interrupt masks container.
#         0 - does not mask coresponding interrupt
#         1 - masks coresponding interrupt
#Return:
#   None

InterruptMask_in = handle.GetStruct_XDfePrach_InterruptMask()
InterruptMask_in['DecimatorOverflow'] = 1
InterruptMask_in['MixerOverflow'] = 1
InterruptMask_in['DecimatorOverrun'] = 1
InterruptMask_in['SelectorOverrun'] = 1
InterruptMask_in['RachUpdate'] = 1
InterruptMask_in['CCSequenceError'][0] = 1
InterruptMask_in['FrameInitTrigger'][0] = 1
InterruptMask_in['FrameError'][0] = 1
handle.XDfePrach_SetInterruptMask(device_id, InterruptMask_in)

#Description:
#   Gets event status.
#C header declaration:
#   void XDfePrach_GetEventStatus(const XDfePrach *InstancePtr, XDfePrach_StatusMask *Mask);
#Input Arguments:
#   device_id: id of the opened device.
#Return:
#   Status: event status container.

EventStatus_out = handle.XDfePrach_GetEventStatus(device_id)

#Description:
#   Gets event status.
#C header declaration:
#   void XDfePrach_ClearEventStatus(const XDfePrach *InstancePtr,
#   const XDfePrach_StatusMask *Status);
#Input Arguments:
#   device_id: id of the opened device.
#Return:
#   Status: event status container.

Status_in = handle.GetStruct_XDfePrach_StatusMask()
Status_in['DecimatorOverflow'] = 1
Status_in['MixerOverflow'] = 1
Status_in['DecimatorOverrun'] = 1
Status_in['SelectorOverrun'] = 1
Status_in['RachUpdate'] = 1
Status_in['CCSequenceError'][0] = 1
Status_in['FrameInitTrigger'][0] = 1
Status_in['FrameError'][0] = 1
handle.XDfePrach_ClearEventStatus(device_id, Status_in)

#Description:
#   Remove a CCID from sequence.
#C header declaration:
#   u32 XDfePrach_RemoveCC(const XDfePrach *InstancePtr, u32 CCId);
#Input Arguments:
#   device_id: id of the opened device.
#   CCID: is a Channel ID.
#Return:
#   ret: XST_SUCCESS if successful, XST_FAILURE if error occurs.

ret = handle.XDfePrach_RemoveCC(device_id, 0)

#Description:
#   Removes an RC configuration entry from the RC_CONFIGURATION.
#C header declaration:
#   u32 XDfePrach_RemoveRC(const XDfePrach *InstancePtr, u32 RCId);
#Input Arguments:
#   device_id: id of the opened device.
#   RCId: is RC Id.
#Return:
#   ret: XST_SUCCESS on success, XST_FAILURE on failure

ret = handle.XDfePrach_RemoveRC(device_id, 0)

#Description:
#   Reads the delay, which will be added to TUSER and TLAST (delay matched
#   through the IP).
#C header declaration:
#   u32 XDfePrach_GetTUserDelay(const XDfePrach *InstancePtr);
#Input Arguments:
#   device_id: id of the opened device.
#Return:
#   ret: Delay value

ret = handle.XDfePrach_GetTUserDelay(device_id)

"""
Reads the delay of specified band in multiband mode, which will be
added to TUSER and TLAST (delay matched through the IP).
:param device_id: id of the opened device.
:param BandId: Band Id of requested delay value.
:return: ret: Delay value
"""
BandId = 0
ret = handle.XDfePrach_GetTUserDelayMB(device_id, BandId)

#Description:
#   Returns CONFIG.DATA_LATENCY.VALUE + tap, where the tap is between 0
#   and 23 in real mode and between 0 and 11 in complex/matrix mode.
#C header declaration:
#   u32 XDfePrach_GetTDataDelay(const XDfePrach *InstancePtr);
#Input Arguments:
#   device_id: id of the opened device.
#Return:
#   ret: data latency value.

ret = handle.XDfePrach_GetTDataDelay(device_id)

"""
Returns data latency of specified band in multiband mode.
:param device_id: id of the opened device.
:param BandId: Band Id of requested delay value.
:return: ret: data latency value.
"""
BandId = 0
ret = handle.XDfePrach_GetTDataDelayMB(device_id, BandId)

#Description:
#   This API gets the driver and HW design version.
#C header declaration:
#   void XDfePrach_GetVersions(const XDfePrach *InstancePtr, XDfePrach_Version *SwVersion,
# 	XDfePrach_Version *HwVersion);
#Input Arguments:
#   device_id: id of the opened device.
#Return:
#   SwVersion: driver version numbers.
#   HwVersion: HW version numbers.

SwVersion_out, HwVersion_out = handle.XDfePrach_GetVersions(device_id)

#Description:
#   Deactivates PRACH and moves the state machine to Initialised state.
#C header declaration:
#   void XDfePrach_Deactivate(XDfePrach *InstancePtr);
#Input Arguments:
#   device_id: id of the opened device.
#Return:
#   None

handle.XDfePrach_Deactivate(device_id)

#Description:
#   Sets the delay, which will be added to TUSER and TLAST (delay matched through the IP).
#C header declaration:
#   u32 XDfePrach_SetTUserDelay(const XDfePrach *InstancePtr, u32 Delay);
#Input Arguments:
#   device_id: id of the opened device.
#   Delay: requested delay variable.
#Return:
#   None

handle.XDfePrach_SetTUserDelay(device_id, 10)

"""
Sets the delay of specified band in multiband mode, which will be added
to TUSER and TLAST (delay matched through the IP).
:param device_id: id of the opened device.
:param Delay: requested delay variable.
:param BandId: Band Id of requested delay value.
:return: None
"""
Delay = 10
BandId = 0
handle.XDfePrach_SetTUserDelayMB(device_id, Delay, BandId)

#Description:
#   API closes the instances of a PRACH driver and moves the state machine to
#   a Not Ready state.
#C header declaration:
#   void XDfePrach_InstanceClose(XDfePrach *InstancePtr);
#Input Arguments:
#   device_id: id of the opened device.
#Return:
#   None

handle.XDfePrach_InstanceClose(device_id)



#!/usr/bin/env python
# Copyright (C) 2023-2025 Advanced Micro Devices, Inc.  All rights reserved.
# SPDX-License-Identifier: BSD-3-Clause

__author__ = "Salih Erim, Sree Parvathi Anish"
__copyright__ = "Copyright 2023-2025, Advanced Micro Devices, Inc."

import os
import sys
import json
import argparse
import csv
import time
import datetime
import operator
sys.path.append('/usr/share/raft/xclient/raft_services')
from pm_client import PM_Client

client = PM_Client()

class CustomHelpFormatter(argparse.HelpFormatter):
    """Custom formatter to show subcommand groups with proper argument types."""
    def __init__(self, prog, command_groups):
        super().__init__(prog)
        self.command_groups = command_groups  # Store command groups

    def _format_action(self, action):
        """Customize argument formatting to distinguish between positional and optional arguments."""
        if isinstance(action, argparse._SubParsersAction):
            parts = []
            for group_name, commands in self.command_groups.items():
                parts.append(f"\n  {group_name}:")
                for cmd in commands:
                    parts.append(f"    {cmd:<15} {action.choices[cmd].description}")
            return "\n".join(parts)
        # Distinguish between positional and optional arguments
        if action.option_strings:
            # Optional arguments (e.g., --flag, --option value)
            return f"  {', '.join(action.option_strings):<20} {action.help}"
        else:
            # Positional arguments (e.g., required ones)
            return f"  {action.dest:<20} {action.help}"
        return super()._format_action(action)

class CheckRange(argparse.Action):
    ops = {'inf': operator.gt,
           'min': operator.ge,
           'sup': operator.lt,
           'max': operator.le}

    def __init__(self, *args, **kwargs):
        if 'min' in kwargs and 'inf' in kwargs:
            raise ValueError('either min or inf, but not both')
        if 'max' in kwargs and 'sup' in kwargs:
            raise ValueError('either max or sup, but not both')
        for name in self.ops:
            if name in kwargs:
                setattr(self, name, kwargs.pop(name))
        super().__init__(*args, **kwargs)

    def interval(self):
        if hasattr(self, 'min'):
            l = f'[{self.min}'
        elif hasattr(self, 'inf'):
            l = f'({self.inf}'
        else:
            l = '(-infinity'

        if hasattr(self, 'max'):
            u = f'{self.max}]'
        elif hasattr(self, 'sup'):
            u = f'{self.sup})'
        else:
            u = '+infinity)'
        return f'valid range: {l}, {u}'

    def __call__(self, parser, namespace, values, option_string=None):
        for name, op in self.ops.items():
            if hasattr(self, name) and not op(values, getattr(self, name)):
                raise argparse.ArgumentError(self, self.interval())
            setattr(namespace, self.dest, values)

def print_response(response):
    if response['status'] == 'success':
        if not response['data']:
            print(json.dumps(response['status'], indent=2))
        else:
            print(json.dumps(response['data'], indent=2))
        sys.exit(os.EX_OK)
    else:
        print(json.dumps(response['message'], indent=2))
        sys.exit(os.EX_SOFTWARE)

def main():
    # Define command groups
    command_groups = {
        "System Information": ["boardinfo", "listfeature"],
        "Power Sensor Measurement and Control": ["listpower", "getpower", "getcalpower", "getinaconf", "setinaconf"],
        "Reagultor Voltage Measurement and Control": ["listvoltage", "enablevoltage", "disablevoltage", "getvoltage", "setvoltage", "setbootvoltage", "restorevoltage", "getregulator"],
        "Power Domains and Rails Info": ["listdomain", "listrail", "powervalue", "allvalue"],
        "Temperature Measurement": ["listtemperature", "gettemperature"],
        "Measurement Units": ["listunit", "getunit", "availablescale", "setscale"],
        "Miscellaneous": ["loglevel", "output-csv"]
    }
    # Create main parser
    parser = argparse.ArgumentParser(
        description="RAFT-PM CLI tool for system monitoring and power management",
        formatter_class=lambda prog: CustomHelpFormatter(prog, command_groups)
    )
    # Create subparsers for commands
    subparsers = parser.add_subparsers(dest="command", required=True, help="Available commands")
    # Function to create subcommands with a description and execution logic
    def add_command(name, description, func, arguments=None, optional_arguments=None):
        cmd_parser = subparsers.add_parser(name, help=description, description=description)
        # Add required arguments
        if arguments:
            for arg_name, arg_options in arguments.items():
                cmd_parser.add_argument(arg_name, **arg_options)
        # Add optional arguments
        if optional_arguments:
            for arg_name, arg_options in optional_arguments.items():
                cmd_parser.add_argument(arg_name, **arg_options)

        cmd_parser.set_defaults(func=func)

    # 🌟 Add commands with arguments
    add_command("boardinfo", "Display board information", system_action)
    add_command("listfeature", "Show available system feature", system_action)

    add_command("listpower", "List all power sensors", power_action)
    add_command("getpower", "Get power sensor value", power_action,
                arguments={"sensor_name": {"type": str, "help": "Specify the power sensor name (e.g., VCCINT, VCCSOC)"}})
    add_command("getcalpower", "Get calibrated power sensor values", power_action,
                arguments={"sensor_name": {"type": str, "help": "Specify the power sensor name (e.g., VCCINT, VCCSOC)"}})
    add_command("getinaconf", "Retrieve INA configuration", power_action,
                arguments={"sensor_name": {"type": str, "help": "Specify the power sensor name (e.g., VCCINT, VCCSOC)"}})
    add_command("setinaconf", "Set INA configuration", power_action,
                arguments={"sensor_name": {"type": str, "help": "Specify the power sensor name (e.g., VCCINT, VCCSOC)"},
                            "value": {"type": lambda arg: [int(token) if token.isdigit() else None if token.lower() == 'x' else token for token in arg.split(' ')], "help": "Set INA configuration value in decimal values e.g., '1 2 3 4'"}})

    add_command("listvoltage", "List available voltage regulators", voltage_action)
    add_command("enablevoltage", "Enable a specific voltage regulator output", voltage_action,
                arguments={"voltage_name": {"type": str, "help": "Voltage rail to enable"}})
    add_command("disablevoltage", "Disable a specific voltage regulator output", voltage_action,
                arguments={"voltage_name": {"type": str, "help": "Voltage rail to disable"}})
    add_command("getvoltage", "Get current voltage level", voltage_action,
                arguments={"voltage_name": {"type": str, "help": "Voltage rail to set"}})
    add_command("setvoltage", "Set a new voltage level", voltage_action,
                arguments={"voltage_name": {"type": str, "help": "Voltage rail to set"},
                           "value": {"type": float, "help": "New voltage value"}})
    add_command("setbootvoltage", "Set boot voltage level", voltage_action,
                arguments={"voltage_name": {"type": str, "help": "Voltage rail to set"},
                           "value": {"type": float, "help": "New voltage value"}})
    add_command("restorevoltage", "Restore default voltage value", voltage_action,
                arguments={"voltage_name": {"type": str, "help": "Voltage rail to set"}})
    add_command("getregulator", "Get ragulators all available telemetry data", voltage_action,
                arguments={"voltage_name": {"type": str, "help": "Voltage rail to get"}})

    add_command("listdomain", "List all power domains", domain_action)
    add_command("listrail", "List all power rails for specific domain", domain_action,
                arguments={"domain_name": {"type": str, "help": "Domain name to get rails"}})
    add_command("powervalue", "Get only 'power' values of all domains", domain_action)
    add_command("allvalue", "Get all values retrieved from power sensors in once", domain_action)

    add_command("listtemperature", "List all tmeperature sensors", temp_action)
    add_command("gettemperature", "Get temperature values for given sensor", temp_action,
                arguments={"temp_name": {"type": str, "help": "temp_name"}})

    add_command("listunit", "List available measurement units", unit_action)
    add_command("getunit", "Get unit details",  unit_action,
                arguments={"quantity": {"type": str, "choices": ["voltage", "current", "power", "temperature"], "help": "quantity"}})
    add_command("availablescale", "Show available scaling options", unit_action,
                arguments={"quantity": {"type": str, "choices": ["voltage", "current", "power", "temperature"], "help": "quantity"}})
    add_command("setscale", "Set measurement scale", unit_action,
                arguments={"quantity": {"type": str, "choices": ["voltage", "current", "power", "temperature"], "help": "quantity"},
                           "scale": {"type": str, "help": "Scaling factor to apply"}})

    add_command("loglevel", "Set logging level", log_action, {
        "level": {"type": str, "choices": ["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"], "help": "Logging level"}
    })
    add_command("output-csv", "Time-series allvalue output results in CSV format", outputcsv_action,
                arguments={"duration": {"type": int, "min": 1, "action": CheckRange, "help": "Duration time value"},
                           "sampling_rate": {"type": int, "min": 1, "max": 12, "action": CheckRange, "help": "Sampling rate value"}},
                optional_arguments={"--path": {"type": str, "default": "/home/petalinux", "help": "Path to save the output file"}})
    # Parse arguments
    args = parser.parse_args()
    # Execute the corresponding function
    if hasattr(args, "func"):
        args.func(args)
    else:
        parser.print_help()

def system_action(args):
    match args.command:
        case 'boardinfo':
            print_response(client.getboardinfo())
        case 'listfeature':
            print_response(client.listfeature())

def power_action(args):
    match args.command:
        case "listpower":
            print_response(client.listpower())
        case "getpower":
            print_response(client.getpower(args.sensor_name))
        case "getcalpower":
            print_response(client.getcalpower(args.sensor_name))
        case "getinaconf":
            print_response(client.getpowerconf(args.sensor_name))
        case "setinaconf":
            print_response(client.setpowerconf(args.sensor_name, args.value))

def voltage_action(args):
    match args.command:
        case "listvoltage":
            print_response(client.listvoltage())
        case "enablevoltage":
            print_response(client.enablevoltage(args.voltage_name))
        case "disablevoltage":
            print_response(client.disablevoltage(args.voltage_name))
        case "getvoltage":
            print_response(client.getvoltage(args.voltage_name))
        case "setvoltage":
            print_response(client.setvoltage(args.voltage_name, args.value))
        case "setbootvoltage":
            print_response(client.setbootvoltage(args.voltage_name, args.value))
        case "restorevoltage":
            print_response(client.restorevoltage(args.voltage_name))
        case "getregulator":
            print_response(client.getregulator(args.voltage_name))

def domain_action(args):
    match args.command:
        case "listdomain":
            print_response(client.listpowerdomain())
        case "listrail":
            print_response(client.listrailsofdomain(args.domain_name))
        case "powervalue":
            print_response(client.getpowerall())
        case "allvalue":
            print_response(client.getvalueall())

def temp_action(args):
    match args.command:
        case 'listtemperature':
            print_response(client.listtemperature())
        case 'gettemperature':
            print_response(client.gettemperature(args.temp_name))

def unit_action(args):
    match args.command:
        case "listunit":
            print_response(client.listunit())
        case "getunit":
            print_response(client.getunit(args.quantity))
        case "availablescale":
            print_response(client.listscale(args.quantity))
        case "setscale":
            print_response(client.setscale(args.quantity, args.scale))

def log_action(args):
    print(client.logger.level)

def outputcsv_action(args):
    sample_rate = int(args.sampling_rate)
    filepath = args.path
    duration = int(args.duration)
    # Check the input arguments
    if not os.path.exists(filepath):
        print(f"The path {filepath} is valid.")
        sys.exit(1)
    # Find the sleep time from the sample rate
    sleeptime = 1/sample_rate
    csv_data = []
    # Mark the start time
    start_datetime = datetime.datetime.now()
    start_time = time.time()
    while True:
        operation_start_time = time.time()
        ret = client.getvalueall()
        if ret["status"] == "success":
            csv_data, columns_order = update_csv_data(csv_data, ret["data"], sample_rate, duration)
        operation_exec_time = time.time() - operation_start_time
        # If operation execution time is higher than sleep time, no need to sleep
        if (sleeptime - operation_exec_time) > 0:
            time.sleep(sleeptime -  operation_exec_time)
        if time.time() - start_time >= duration:
            break
    filename = start_datetime.strftime(f"pmtool_s-{sample_rate}_d-{duration}_%Y-%m-%d_%H-%M-%S.csv")
    pm_csv_dump(csv_data, columns_order, filepath, filename)

def update_csv_data(csv_data, data, sample_rate, duration):
    # Convert the dictionary to a list
    columns_order = []
    row_item = {}
    write_timestamp = 1
    for k, domain_list in data.items():
        for index, domain in enumerate(domain_list):
            for key, value in domain.items():
                for rails_item in value["Rails"]:
                    for rails_key, rails_value in rails_item.items():
                        if write_timestamp == 1:
                            columns_order.append("Timestamp")
                            start_time = datetime.datetime.now()
                            row_item.update({"Timestamp" : start_time})
                            write_timestamp = 0
                        column_name = f"{key}-{rails_key}-{list(rails_value.keys())[0]}"
                        columns_order.append(column_name)
                        row_item.update({f"{column_name}": rails_value['Voltage']})
                        column_name = f"{key}-{rails_key}-{list(rails_value.keys())[1]}"
                        columns_order.append(column_name)
                        row_item.update({f"{column_name}": rails_value['Current']})
                        column_name = f"{key}-{rails_key}-{list(rails_value.keys())[2]}"
                        columns_order.append(column_name)
                        row_item.update({f"{column_name}": rails_value['Power']})
        column_name = f"{key}-TotalPower"
        columns_order.append(column_name)
        row_item.update({f"{column_name}": value["Total Power"]})
    csv_data.append(row_item)
    return csv_data, columns_order

def pm_csv_dump(csv_data, columns_order, filepath, filename):
    csv_file = f"{filepath}/{filename}"
    try:
        with open(csv_file, 'w') as csvfile:
            csv_writer = csv.DictWriter(csvfile, fieldnames=columns_order, lineterminator='\n')
            csv_writer.writeheader()
            for row in csv_data:
                    csv_writer.writerow(row)
            print(f"The requested data is written to file - {csv_file}")
            sys.exit(os.EX_OK)
    except IOError:
        print(f"I/O error while opening {csv_file} to write")
        sys.exit(os.EX_SOFTWARE)

if __name__ == "__main__":
    # call main
    main()




# Copyright (C) 2024-2025 Advanced Micro Devices, Inc.  All rights reserved.
# SPDX-License-Identifier: BSD-3-Clause

__author__ = "Salih Erim"
__copyright__ = "Copyright 2024-2025, Advanced Micro Devices, Inc."

import json
from enum import IntEnum

class Sysmon_Device_Type(IntEnum):
    I2C = 0
    PMBUS = 1
    SYSFS = 2

class Rails:
    def __init__(self, Name, Part_Name, I2C_Bus, I2C_Address,
                 Shunt_Resistor=None, Maximum_Current=None, Phase_Multiplier=None,
                 Maximum_Volt=None, Typical_Volt=None, Minimum_Volt=None,
                 PMBus_VOUT_MODE=-1, Page_Select=-1, Phase=-1, FB_Ratio=1.0, Voltage_Multiplier=-1):
        self.name = Name
        self.part_name = Part_Name
        self.i2c_bus = I2C_Bus
        self.i2c_address = I2C_Address
        self.shunt_resistor = Shunt_Resistor
        self.maximum_current = Maximum_Current
        self.phase_multiplier = Phase_Multiplier
        self.maximum_volt = Maximum_Volt
        self.typical_volt = Typical_Volt
        self.minimum_volt = Minimum_Volt
        self.pmbus_vout_mode = int(PMBus_VOUT_MODE)
        self.page_select = int(Page_Select)
        self.phase = int(Phase)
        self.fb_ratio = float(FB_Ratio)
        self.voltage_multiplier = int(Voltage_Multiplier)

        self._sensor = None
        self._output = None

    def __str__(self):
        str_info = {
            k: v
            for k, v in self.__dict__.items()
            if k and v is not None
        }
        return str(str_info)

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
            sort_keys=True, indent=4)

class Domain:
    def __init__(self, Name, Rails):
        self.name = Name
        self.railnames = Rails

    def __str__(self):
        str_info = {
            k: v
            for k, v in self.__dict__.items()
            if k and v
        }
        return str(str_info)

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
            sort_keys=True, indent=4)

class Unit:
    def __init__(self, base_unit, available_scales):
        self.base_unit = base_unit
        self.available_scales = available_scales
        self.current_unit = None
        self.quantity = None

    def __str__(self):
        str_info = {
            k: v
            for k, v in self.__dict__.items()
            if k and v
        }
        return str(str_info)


# Copyright (C) 2023-2025 Advanced Micro Devices, Inc.  All rights reserved.
# SPDX-License-Identifier: BSD-3-Clause

__author__ = "Salih Erim"
__copyright__ = "Copyright 2023-2025, Advanced Micro Devices, Inc."

import Pyro4
import Pyro4.utils
import logging
import base64
import json
import sys

sys.excepthook = Pyro4.util.excepthook

class PM_Client(object):
    PM = None
    logger = None
    LogLevelsDict = {
        "DEBUG": 4,
        "INFO": 3,
        "WARNING": 2,
        "ERROR": 1,
        "CRITICAL": 0
    }

    def __init__(self):
        self.logger = self.GetLogger()
        self.logger.info("Inside Raft-PM Client Constructor")
        self.SetIpAndPort("127.0.0.1", "9090")
        return

    @staticmethod
    def GetLogger():
        """
        Static method to get the logger for the class.
        Default loglevel is set inside this class

        :return: logger
        """
        log_level = logging.ERROR
        logging.basicConfig(format="%(levelname)s:%(message)s")
        logger = logging.getLogger(__name__)
        try:
            handler_set_check = getattr(logger, 'handler_set')
        except AttributeError:
            handler_set_check = False
        if not handler_set_check:
            logger.setLevel(log_level)
            handler = logging.StreamHandler(sys.stdout)
            logger.addHandler(handler)
            logger.handler_set = True
            logger.disabled = False
        return logger

    def SetIpAndPort(self, ipaddr, port):
        """
        API to inform PM Client the IP address and port number of PM Server.

        :param ipaddr: IP Address string
        :param port: Port number string
        :return: None
        """
        uri = f"PYRO:PM@{ipaddr}:{port}"
        self.logger.debug(f"SetIpAndPort({ipaddr}, {port})\n uri = {uri}")
        self.PM = Pyro4.Proxy(uri)
        pass

    def GetPythonLogLevels(self):
        """
        Return the logging levels supported by logging library in python

        :return: Dictionary showing the log levels supported by logging library
        """
        self.logger.debug("GetPythonLogLevels()")
        PythonLogLevel = self.PM.GetPythonLogLevels()
        self.logger.debug(f"PythonLogLevel = {json.dumps(PythonLogLevel, indent=2)}")
        return PythonLogLevel

    def SetServerLogLevel(self, PythonLogLevel):
        """
        Set the python log level to the given level

        :param PythonLogLevel: Log level to set
        :return: None
        """
        self.logger.debug(f"SetServerLogLevel({PythonLogLevel})")
        self.PM.SetServerLogLevel(PythonLogLevel)
        return

    def SetClientLogLevel(self, PythonLogLevel):
        """
        Set the python log level to the given level

        :param PythonLogLevel: Log level to set
        :return: None
        """
        if PythonLogLevel == self.LogLevelsDict["DEBUG"]:
            self.logger.setLevel(logging.DEBUG)
        elif PythonLogLevel == self.LogLevelsDict["INFO"]:
            self.logger.setLevel(logging.INFO)
        elif PythonLogLevel == self.LogLevelsDict["WARNING"]:
            self.logger.setLevel(logging.WARNING)
        elif PythonLogLevel == self.LogLevelsDict["ERROR"]:
            self.logger.setLevel(logging.ERROR)
        else:
            self.logger.setLevel(logging.CRITICAL)
        return

    def getboardinfo(self):
        """
        Gets Board's Info

        :param : None
        :return: Board Info
        """
        self.logger.debug("getboardinfo()")
        return self.PM.GetBoardInfo()

    def listfeature(self):
        """
        Gets feature list

        :param : None
        :return: Feature List
        """
        self.logger.debug("listfeature()")
        return self.PM.ListFeature()

    def listpowerdomain(self):
        """
        Gets list of Power Domains.

        :param : None
        :return: Domains
        """
        self.logger.debug("listpowerdomain()")
        return self.PM.ListPowerDomains()

    def listrailsofdomain(self, domainname):
        """
        Gets list of Rails given domain name.

        :param domainname: string of a "domainname"
        :return: Rails name list
        """
        self.logger.debug("listrailsofdomain()")
        return self.PM.ListRailsOfDomain(domainname)

    def getvalueofrail(self, railname):
        """
        Gets list of the rail's sensor values given rail name.

        :param railname: string of a "railname"
        :return: Sensor values of the Rail
        """
        self.logger.debug(f"getvalueofrail({railname})")
        return self.PM.GetPowerSensor(railname)

    def getvalueofdomain(self, domainname):
        """
        Gets the domain's all rail sensor values given domain name.

        :param : string of a "domainname"
        :return: The domain's all rails sensor values of the Rail
        """
        self.logger.debug("getvalueofdomain({domainname})")
        return self.PM.GetValueOfDomain(domainname)

    def getpowerall(self):
        """
        Gets the boards's all domain's and total power values

        :param : None
        :return: The boards's all domain's and total power values
        """
        self.logger.debug("getpowerall()")
        return self.PM.GetPowersAll()

    def getvalueall(self):
        """
        Gets the boards's all domain's rails sensor values

        :param : None
        :return: The board's all rails sensor values of the Rail
        """
        self.logger.debug("getvalueall()")
        return self.PM.GetValuesAll()

    def listtemperature(self):
        """
        Gets the Sysmon temperature values

        :param : None
        :return: The sysmon temperature values
        """
        self.logger.debug("listtemperature()")
        return self.PM.ListTemperatures()

    def gettemperature(self, name):
        """
        Gets the Sysmon temperature values

        :param : None
        :return: The sysmon temperature values
        """
        self.logger.debug("GetSysmonTemperatures()")
        return self.PM.GetTemperature(name)

    def listpower(self):
        self.logger.debug("ListPowerSensors()")
        return self.PM.ListPowerSensors()

    def getpower(self, name):
        self.logger.debug("GetPowerSensor()")
        return self.PM.GetPowerSensor(name)

    def getcalpower(self, name):
        self.logger.debug("GetCalPowerSensor()")
        return self.PM.GetCalPowerSensor(name)

    def getpowerconf(self, name):
        self.logger.debug("GetPowerSensorConf()")
        return self.PM.GetPowerSensorConf(name)

    def setpowerconf(self, name, conf):
        self.logger.debug("SetPowerSensorConf()")
        return self.PM.SetPowerSensorConf(name, conf)

    def listvoltage(self):
        self.logger.debug("ListVoltages()")
        return self.PM.ListVoltages()

    def enablevoltage(self, name):
        self.logger.debug("EnableVoltage()")
        return self.PM.EnableVoltage(name)

    def disablevoltage(self, name):
        self.logger.debug("DisableVoltage()")
        return self.PM.DisableVoltage(name)

    def getregulator(self, name):
        self.logger.debug("GetRegulator()")
        return self.PM.GetRegulator(name)

    def getvoltage(self, name):
        self.logger.debug("GetVoltage()")
        return self.PM.GetVoltage(name)

    def setvoltage(self, name, value):
        self.logger.debug("SetVoltage()")
        return self.PM.SetVoltage(name, value)

    def setbootvoltage(self, name, value):
        """
        Set boot voltage for the regulator.

        :name : voltage regulator name
        :value : voltage value in volt
        :return: Success mesage
        """
        self.logger.debug("SetBootVoltage()")
        return self.PM.SetBootVoltage(name, value)

    def restorevoltage(self, name):
        """
        Reset to typical voltage.

        :name : voltage output name
        :return: Success mesage
        """
        self.logger.debug("RestoreVoltage()")
        return self.PM.RestoreVoltage(name)

    def listunit(self):
        self.logger.debug("ListUnits()")
        return self.PM.ListUnits()

    def getunit(self, quantity):
        self.logger.debug("GetUnit()")
        return self.PM.GetUnit(quantity)

    def setscale(self, quantity, scale):
        self.logger.debug("SetScale()")
        return self.PM.SetScale(quantity, scale)

    def listscale(self, quantity):
        self.logger.debug("ListAvailableScales()")
        return self.PM.ListAvailableScales(quantity)

pm = PM_Client()